@@ -11,11 +11,11 @@ namespace hannk {
 class Add : public Generator<Add> {
 public:
     // Input buffers and quantization parameters.
-    Input<Buffer<uint8_t>> input1_{"input1", 2};
+    Input<Buffer<uint8_t, 2>> input1_{"input1"};
     Input<uint8_t> input1_zero_{"input1_zero"};
     Input<int16_t> input1_multiplier_{"input1_multiplier"};
 
-    Input<Buffer<uint8_t>> input2_{"input2", 2};
+    Input<Buffer<uint8_t, 2>> input2_{"input2"};
     Input<uint8_t> input2_zero_{"input2_zero"};
     Input<int16_t> input2_multiplier_{"input2_multiplier"};
 
@@ -23,7 +23,7 @@ public:
     Input<uint8_t> output_min_{"output_min"};
     Input<uint8_t> output_max_{"output_max"};
 
-    Output<Buffer<uint8_t>> output_{"output", 2};
+    Output<Buffer<uint8_t, 2>> output_{"output"};
 
     void generate() {
         Var x("x"), y("y");
@@ -61,10 +61,10 @@ public:
 
 class Mul : public Generator<Mul> {
 public:
-    Input<Buffer<uint8_t>> input1_{"input1", 2};
+    Input<Buffer<uint8_t, 2>> input1_{"input1"};
     Input<uint8_t> input1_zero_{"input1_zero"};
 
-    Input<Buffer<uint8_t>> input2_{"input2", 2};
+    Input<Buffer<uint8_t, 2>> input2_{"input2"};
     Input<uint8_t> input2_zero_{"input2_zero"};
 
     Input<uint8_t> output_zero_{"output_zero"};
@@ -73,7 +73,7 @@ public:
     Input<uint8_t> output_min_{"output_min"};
     Input<uint8_t> output_max_{"output_max"};
 
-    Output<Buffer<uint8_t>> output_{"output", 2};
+    Output<Buffer<uint8_t, 2>> output_{"output"};
 
     void generate() {
         Var x("x"), y("y");
@@ -122,12 +122,15 @@ public:
     GeneratorParam<Type> output3_type_{"output3_type", Int(0)};
 
     // An array of inputs.
-    Input<Buffer<>[]> inputs_ { "inputs", 2 };
+    Input<Buffer<void, 2>[]> inputs_ { "inputs" };
     // The program to run. See elementwise_program.h for a description of
     // this buffer.
-    Input<Buffer<int16_t>> program_{"program", 2};
+    Input<Buffer<int16_t, 2>> program_{"program"};
 
-    Func build() {
+    // Type is determined by the GeneratorParams specified.
+    Output<Buffer<void, 2>> output_{"output"};
+
+    void generate() {
         Var x("x"), y("y"), u("u");
 
         Type intermediate_type = intermediate_type_;
@@ -175,7 +178,6 @@ public:
         r.where(r.x == op);
         scratch(x, y, slot) = mux(r.x, instructions);
 
-        Func output("output");
         std::vector<Type> output_types;
         if (((Type)output1_type_).bits() > 0) {
             output_types.push_back(output1_type_);
@@ -195,10 +197,10 @@ public:
             output_i = saturating_cast(output_types[i], output_i);
             outputs.push_back(output_i);
         }
-        output(x, y) = Tuple(outputs);
+        output_(x, y) = Tuple(outputs);
 
         // Schedule.
-        output.compute_root()
+        output_.compute_root()
             .vectorize(x, natural_vector_size<uint8_t>(), TailStrategy::Predicate);
 
         // Only allow this many instructions per input, so we can store scratch
@@ -222,8 +224,6 @@ public:
 
         program_.dim(0).set_min(0).set_extent(ElementwiseAssembler::InstructionSize).set_stride(1);
         program_.dim(1).set_min(0).set_stride(ElementwiseAssembler::InstructionSize);
-
-        return output;
     }
 };
 
