@@ -7,15 +7,15 @@
 
 #include <map>
 #include <set>
+#include <unordered_set>
 
 // TODO: apparently not part of the public Halide API. Should it be?
 extern "C" int halide_malloc_alignment();
 
 namespace hannk {
 
-Interpreter::Interpreter(std::unique_ptr<OpGroup> m, InterpreterOptions options)
-    : model_(std::move(m)) {
-    init(options);
+Interpreter::Interpreter(OpPtr m, InterpreterOptions options)
+    : model_(std::move(m)), options_(std::move(options)) {
 }
 
 Interpreter::~Interpreter() {
@@ -25,12 +25,14 @@ namespace {
 
 // TODO: maybe move this to a separate file? Not sure if it's complex enough to be worthy or not.
 class TensorVisitor : public OpVisitor {
+    using OpVisitor::visit;
+
     virtual void visit_tensor(const TensorPtr &t) = 0;
 
-    void visit(OpGroup *g) override {
+    void visit(const OpGroup *g) override {
         for (int i = 0; i < g->op_count(); i++) {
             op_index_++;
-            Op *op = g->op(i);
+            const Op *op = g->op(i);
             for (int j = 0; j < op->input_count(); j++) {
                 visit_tensor(op->input(j));
             }
@@ -87,7 +89,7 @@ public:
     std::map<TensorStoragePtr, TensorAllocationInfo> tensor_info;
 };
 
-std::unique_ptr<char[]> allocate_tensors(OpGroup *root, const InterpreterOptions &options) {
+std::unique_ptr<char[]> allocate_tensors(const Op *root, const InterpreterOptions &options) {
     // Find the tensors that we want to allocate in an arena,
     // along the needed storage size and lifetime for each.
     FindAllocatableTensors find_tensors;
@@ -160,52 +162,195 @@ class VerifyAllAllocated : public TensorVisitor {
     }
 };
 
+#ifndef NDEBUG
+
+class Checker : public OpVisitor {
+    using OpVisitor::visit;
+
+    std::unordered_set<Tensor *> valid_tensors_;
+
+    void check_tensors(const Op *op) {
+        for (int j = 0; j < op->input_count(); j++) {
+            Tensor *t = op->input(j).get();
+            if (!t->is_constant() && !valid_tensors_.count(t)) {
+                HLOG(ERROR) << "Op " << op->name() << " uses tensor " << op->input(j)->name() << " but it is not produced yet\n";
+                correct = false;
+                return;
+            }
+        }
+        for (int j = 0; j < op->output_count(); j++) {
+            valid_tensors_.insert(op->output(j).get());
+        }
+    }
+
+    void visit_leaf(const Op *op) override {
+        if (!correct) {
+            return;
+        }
+        check_tensors(op);
+    }
+
+    void visit(const OpGroup *op) override {
+        if (!correct) {
+            return;
+        }
+        check_tensors(op);
+        OpVisitor::visit(op);
+    }
+
+public:
+    explicit Checker(const Op *root) {
+        for (int j = 0; j < root->input_count(); j++) {
+            valid_tensors_.insert(root->input(j).get());
+        }
+    }
+
+    bool correct = true;
+};
+
+// Verify that no Op comes before any of its input Tensors are produced.
+void do_check_op_order(const Op *root) {
+    Checker checker(root);
+    root->accept(&checker);
+    if (!checker.correct) {
+        HCHECK(0) << "The model is not in the correct order.";
+    }
+}
+#endif
+
 }  // namespace
 
-void Interpreter::init(InterpreterOptions options) {
-    pad_for_ops(model_.get());
-    in_place(model_.get());
-    fold_constants(model_.get());
-    remove_dead_ops(model_.get());
+bool Interpreter::prepare() {
+    if (prepared_) {
+        HLOG(ERROR) << "Do not call prepare() twice";
+        return false;
+    }
 
+    // We must prepare the model before doing the transforms, as some of the
+    // transforms may rely on information cached by prepare(), e.g. alignment requirements.
+    // (Note that any transforms that add new ops are expected to call prepare() on them,
+    // returning errors as appropriate.)
+    if (!model_->prepare()) {
+        HLOG(ERROR) << "model_->prepare() failed.";
+        return false;
+    }
+
+    const auto dump_model = [this](const char *msg, int min_verbosity) {
+        if (options_.verbosity >= min_verbosity) {
+            std::ostringstream os;
+            os << msg << "\n";
+            model_->dump(os);
+            HLOG(INFO) << os.str();
+        }
+    };
+
+    dump_model("Model after prepare():", 3);
+
+    model_ = pad_for_ops(std::move(model_));
+    if (!model_) {
+        HLOG(ERROR) << "pad_for_ops() failed.";
+        return false;
+    }
+    dump_model("Model after pad_for_ops():", 3);
+
+    model_ = in_place(std::move(model_));
+    dump_model("Model after in_place():", 3);
+
+    model_ = fold_constants(std::move(model_));
+    dump_model("Model after fold_constants():", 3);
+
+    model_ = flatten_groups(std::move(model_));
+    dump_model("Model after flatten_groups:", 3);
+
+    model_ = fuse_pad_ops(std::move(model_));
+    if (!model_) {
+        HLOG(ERROR) << "fuse_pad_ops() failed.";
+        return false;
+    }
+    dump_model("Model after fuse_pad_ops:", 3);
+
+    model_ = remove_dead_ops(std::move(model_));
+    dump_model("Model after remove_dead_ops:", 3);
+
+#ifndef NDEBUG
+    do_check_op_order(model_.get());
+#endif
     assert(tensor_storage_arena_ == nullptr);
-    tensor_storage_arena_ = allocate_tensors(model_.get(), options);
+    tensor_storage_arena_ = allocate_tensors(model_.get(), options_);
 
 #ifndef NDEBUG
     VerifyAllAllocated verify_all;
     model_->accept(&verify_all);
 #endif
 
-    if (options.verbosity >= 2) {
-        std::ostringstream os;
-        os << "Model after transformations:\n";
-        model_->dump(os);
-        HLOG(INFO) << os.str();
-    }
+    dump_model("Model after all transformations:", 2);
+
+    prepared_ = true;
+    return true;
 }
 
 void Interpreter::execute() {
+    if (!prepared_) {
+        HLOG(ERROR) << "Must call prepare() before execute()";
+        return;
+    }
     model_->execute();
 }
 
 TensorPtr Interpreter::get_tensor(const std::string &name) {
-    for (int i = 0; i < model_->op_count(); i++) {
-        Op *op = model_->op(i);
-        for (int j = 0; j < op->input_count(); j++) {
-            if (op->input(j)->name() == name) {
-                return op->input(j);
+    HCHECK(prepared_);
+
+    class Finder : public OpVisitor {
+        using OpVisitor::visit;
+
+        bool find_tensor(const Op *op) {
+            if (result) {
+                return true;
+            }
+            for (int j = 0; j < op->input_count(); j++) {
+                if (op->input(j)->name() == name_) {
+                    result = op->input(j);
+                    return true;
+                }
+            }
+            for (int j = 0; j < op->output_count(); j++) {
+                if (op->output(j)->name() == name_) {
+                    result = op->output(j);
+                    return true;
+                }
+            }
+            return false;
+        }
+
+        void visit_leaf(const Op *op) override {
+            if (find_tensor(op)) {
+                return;
             }
         }
-        for (int j = 0; j < op->output_count(); j++) {
-            if (op->output(j)->name() == name) {
-                return op->output(j);
+
+        void visit(const OpGroup *op) override {
+            if (find_tensor(op)) {
+                return;
             }
+            OpVisitor::visit(op);
         }
-    }
-    return nullptr;
+
+        const std::string &name_;
+
+    public:
+        explicit Finder(const std::string &name)
+            : name_(name) {
+        }
+        TensorPtr result = nullptr;
+    };
+
+    Finder finder(name);
+    model_->accept(&finder);
+    return finder.result;
 }
 
 std::vector<TensorPtr> Interpreter::inputs() {
+    HCHECK(prepared_);
     std::vector<TensorPtr> result;
     for (int i = 0; i < model_->input_count(); i++) {
         result.push_back(model_->input(i));
@@ -215,6 +360,7 @@ std::vector<TensorPtr> Interpreter::inputs() {
 }
 
 std::vector<TensorPtr> Interpreter::outputs() {
+    HCHECK(prepared_);
     std::vector<TensorPtr> result;
     for (int i = 0; i < model_->output_count(); i++) {
         result.push_back(model_->output(i));
