@@ -6,8 +6,8 @@
 namespace {
 
 struct LinearBlur : public Halide::Generator<LinearBlur> {
-    Input<Buffer<float>> input{"input", 3};
-    Output<Buffer<float>> output{"output", 3};
+    Input<Buffer<float, 3>> input{"input"};
+    Output<Buffer<float, 3>> output{"output"};
 
     void generate() {
         Var x("x"), y("y"), c("c");
