@@ -1,4 +1,3 @@
-
 import array
 import bit
 import sys
@@ -19,12 +18,5 @@ def test():
         print("Expected Exception not raised.", file=sys.stderr)
         exit(1)
 
-
 if __name__ == "__main__":
-    if "darwin" in sys.platform:
-        # Don't run this test on Mac OS X.  It causes sporadic errors of the form
-        # "dyld: termination function 0x108fa45b0 not in mapped image bit.so".
-        # TODO: investigate why this is the case.
-        pass
-    else:
-        test()
+    test()
