@@ -43,7 +43,7 @@ def main():
             assert os.path.isfile(os.path.join(tmpdir, "f_all.o"))
 
         p = os.path.join(tmpdir, "f.html")
-        f.compile_to({hl.Output.stmt_html: p}, args, "f")
+        f.compile_to({hl.OutputFileType.stmt_html: p}, args, "f")
         assert os.path.isfile(p)
 
     finally:
