@@ -5,11 +5,11 @@ namespace PythonBindings {
 
 namespace {
 
-void halide_python_error(void *, const char *msg) {
+void halide_python_error(JITUserContext *, const char *msg) {
     throw Error(msg);
 }
 
-void halide_python_print(void *, const char *msg) {
+void halide_python_print(JITUserContext *, const char *msg) {
     py::gil_scoped_acquire acquire;
     py::print(msg, py::arg("end") = "");
 }
@@ -17,7 +17,7 @@ void halide_python_print(void *, const char *msg) {
 class HalidePythonCompileTimeErrorReporter : public CompileTimeErrorReporter {
 public:
     void warning(const char *msg) override {
-        py::print(msg, py::arg("end") = "");
+        halide_python_print(nullptr, msg);
     }
 
     void error(const char *msg) override {
@@ -32,7 +32,7 @@ void define_error(py::module &m) {
     static HalidePythonCompileTimeErrorReporter reporter;
     set_custom_compile_time_error_reporter(&reporter);
 
-    Halide::Internal::JITHandlers handlers;
+    Halide::JITHandlers handlers;
     handlers.custom_error = halide_python_error;
     handlers.custom_print = halide_python_print;
     Halide::Internal::JITSharedRuntime::set_default_handlers(handlers);
