@@ -8,9 +8,11 @@
 
 namespace Halide {
 
-template<typename T = void>
+template<typename T = void, int Dims = Halide::Runtime::AnyDims>
 class Buffer;
 
+struct JITUserContext;
+
 namespace Internal {
 
 struct BufferContents {
@@ -106,20 +108,23 @@ std::string buffer_type_name() {
 /** A Halide::Buffer is a named shared reference to a
  * Halide::Runtime::Buffer.
  *
- * A Buffer<T1> can refer to a Buffer<T2> if T1 is const whenever T2
- * is const, and either T1 = T2 or T1 is void. A Buffer<void> can
+ * A Buffer<T1, D> can refer to a Buffer<T2, D> if T1 is const whenever T2
+ * is const, and either T1 = T2 or T1 is void. A Buffer<void, D> can
  * refer to any Buffer of any non-const type, and the default
  * template parameter is T = void.
+ *
+ * A Buffer<T, D1> can refer to a Buffer<T, D2> if D1 == D2,
+ * or if D1 is AnyDims (meaning "dimensionality is checked at runtime, not compiletime").
  */
-template<typename T>
+template<typename T, int Dims>
 class Buffer {
     Internal::IntrusivePtr<Internal::BufferContents> contents;
 
-    template<typename T2>
+    template<typename T2, int D2>
     friend class Buffer;
 
-    template<typename T2>
-    static void assert_can_convert_from(const Buffer<T2> &other) {
+    template<typename T2, int D2>
+    static void assert_can_convert_from(const Buffer<T2, D2> &other) {
         if (!other.defined()) {
             // Avoid UB of deferencing offset of a null contents ptr
             static_assert((!std::is_const<T2>::value || std::is_const<T>::value),
@@ -129,22 +134,28 @@ class Buffer {
                               std::is_void<T>::value ||
                               std::is_void<T2>::value,
                           "type mismatch constructing Buffer");
+            static_assert(Dims == AnyDims || D2 == AnyDims || Dims == D2,
+                          "Can't convert from a Buffer with static dimensionality to a Buffer with different static dimensionality");
         } else {
             // Don't delegate to
             // Runtime::Buffer<T>::assert_can_convert_from. It might
-            // not assert is NDEBUG is defined. user_assert is
+            // not assert if NDEBUG is defined. user_assert is
             // friendlier anyway because it reports line numbers when
             // debugging symbols are found, it throws an exception
             // when exceptions are enabled, and we can print the
             // actual types in question.
-            user_assert(Runtime::Buffer<T>::can_convert_from(*(other.get())))
+            using BufType = Runtime::Buffer<T, Dims>;  // alias because commas in user_assert() macro confuses compiler
+            user_assert(BufType::can_convert_from(*(other.get())))
                 << "Type mismatch constructing Buffer. Can't construct Buffer<"
-                << Internal::buffer_type_name<T>() << "> from Buffer<"
-                << type_to_c_type(other.type(), false) << ">\n";
+                << Internal::buffer_type_name<T>() << ", " << Dims << "> from Buffer<"
+                << type_to_c_type(other.type(), false) << ", " << D2 << ">, dimensions() = " << other.dimensions() << "\n";
         }
     }
 
 public:
+    static constexpr int AnyDims = Halide::Runtime::AnyDims;
+    static_assert(Dims == AnyDims || Dims >= 0);
+
     typedef T ElemType;
 
     // This class isn't final (and is subclassed from the Python binding
@@ -164,22 +175,22 @@ public:
     Buffer &operator=(Buffer &&) noexcept = default;
 
     /** Make a Buffer from a Buffer of a different type */
-    template<typename T2>
-    Buffer(const Buffer<T2> &other)
+    template<typename T2, int D2>
+    Buffer(const Buffer<T2, D2> &other)
         : contents(other.contents) {
         assert_can_convert_from(other);
     }
 
     /** Move construct from a Buffer of a different type */
-    template<typename T2>
-    Buffer(Buffer<T2> &&other) noexcept {
+    template<typename T2, int D2>
+    Buffer(Buffer<T2, D2> &&other) noexcept {
         assert_can_convert_from(other);
         contents = std::move(other.contents);
     }
 
     /** Construct a Buffer that captures and owns an rvalue Runtime::Buffer */
-    template<int D>
-    Buffer(Runtime::Buffer<T, D> &&buf, const std::string &name = "")
+    template<int D2>
+    Buffer(Runtime::Buffer<T, D2> &&buf, const std::string &name = "")
         : contents(new Internal::BufferContents) {
         contents->buf = std::move(buf);
         if (name.empty()) {
@@ -198,50 +209,50 @@ public:
              typename = typename std::enable_if<Internal::all_ints_and_optional_name<Args...>::value>::type>
     explicit Buffer(Type t,
                     int first, Args... rest)
-        : Buffer(Runtime::Buffer<T>(t, Internal::get_shape_from_start_of_parameter_pack(first, rest...)),
+        : Buffer(Runtime::Buffer<T, Dims>(t, Internal::get_shape_from_start_of_parameter_pack(first, rest...)),
                  Internal::get_name_from_end_of_parameter_pack(rest...)) {
     }
 
     explicit Buffer(const halide_buffer_t &buf,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(buf), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(buf), name) {
     }
 
     template<typename... Args,
              typename = typename std::enable_if<Internal::all_ints_and_optional_name<Args...>::value>::type>
     explicit Buffer(int first, Args... rest)
-        : Buffer(Runtime::Buffer<T>(Internal::get_shape_from_start_of_parameter_pack(first, rest...)),
+        : Buffer(Runtime::Buffer<T, Dims>(Internal::get_shape_from_start_of_parameter_pack(first, rest...)),
                  Internal::get_name_from_end_of_parameter_pack(rest...)) {
     }
 
     explicit Buffer(Type t,
                     const std::vector<int> &sizes,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(t, sizes), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(t, sizes), name) {
     }
 
     explicit Buffer(Type t,
                     const std::vector<int> &sizes,
                     const std::vector<int> &storage_order,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(t, sizes, storage_order), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(t, sizes, storage_order), name) {
     }
 
     explicit Buffer(const std::vector<int> &sizes,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(sizes), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(sizes), name) {
     }
 
     explicit Buffer(const std::vector<int> &sizes,
                     const std::vector<int> &storage_order,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(sizes, storage_order), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(sizes, storage_order), name) {
     }
 
     template<typename Array, size_t N>
     explicit Buffer(Array (&vals)[N],
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(vals), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(vals), name) {
     }
 
     template<typename... Args,
@@ -249,7 +260,7 @@ public:
     explicit Buffer(Type t,
                     Internal::add_const_if_T_is_const<T, void> *data,
                     int first, Args &&...rest)
-        : Buffer(Runtime::Buffer<T>(t, data, Internal::get_shape_from_start_of_parameter_pack(first, rest...)),
+        : Buffer(Runtime::Buffer<T, Dims>(t, data, Internal::get_shape_from_start_of_parameter_pack(first, rest...)),
                  Internal::get_name_from_end_of_parameter_pack(rest...)) {
     }
 
@@ -259,28 +270,28 @@ public:
                     Internal::add_const_if_T_is_const<T, void> *data,
                     const std::vector<int> &sizes,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(t, data, sizes, name)) {
+        : Buffer(Runtime::Buffer<T, Dims>(t, data, sizes, name)) {
     }
 
     template<typename... Args,
              typename = typename std::enable_if<Internal::all_ints_and_optional_name<Args...>::value>::type>
     explicit Buffer(T *data,
                     int first, Args &&...rest)
-        : Buffer(Runtime::Buffer<T>(data, Internal::get_shape_from_start_of_parameter_pack(first, rest...)),
+        : Buffer(Runtime::Buffer<T, Dims>(data, Internal::get_shape_from_start_of_parameter_pack(first, rest...)),
                  Internal::get_name_from_end_of_parameter_pack(rest...)) {
     }
 
     explicit Buffer(T *data,
                     const std::vector<int> &sizes,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(data, sizes), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(data, sizes), name) {
     }
 
     explicit Buffer(Type t,
                     Internal::add_const_if_T_is_const<T, void> *data,
                     const std::vector<int> &sizes,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(t, data, sizes), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(t, data, sizes), name) {
     }
 
     explicit Buffer(Type t,
@@ -288,66 +299,60 @@ public:
                     int d,
                     const halide_dimension_t *shape,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(t, data, d, shape), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(t, data, d, shape), name) {
     }
 
     explicit Buffer(T *data,
                     int d,
                     const halide_dimension_t *shape,
                     const std::string &name = "")
-        : Buffer(Runtime::Buffer<T>(data, d, shape), name) {
+        : Buffer(Runtime::Buffer<T, Dims>(data, d, shape), name) {
     }
 
-    static Buffer<T> make_scalar(const std::string &name = "") {
-        return Buffer<T>(Runtime::Buffer<T>::make_scalar(), name);
+    static Buffer<T, Dims> make_scalar(const std::string &name = "") {
+        return Buffer<T, Dims>(Runtime::Buffer<T, Dims>::make_scalar(), name);
     }
 
     static Buffer<> make_scalar(Type t, const std::string &name = "") {
         return Buffer<>(Runtime::Buffer<>::make_scalar(t), name);
     }
 
-    static Buffer<T> make_scalar(T *data, const std::string &name = "") {
-        return Buffer<T>(Runtime::Buffer<T>::make_scalar(data), name);
+    static Buffer<T, Dims> make_scalar(T *data, const std::string &name = "") {
+        return Buffer<T, Dims>(Runtime::Buffer<T, Dims>::make_scalar(data), name);
     }
 
-    static Buffer<T> make_interleaved(int width, int height, int channels, const std::string &name = "") {
-        return Buffer<T>(Runtime::Buffer<T>::make_interleaved(width, height, channels),
-                         name);
+    static Buffer<T, Dims> make_interleaved(int width, int height, int channels, const std::string &name = "") {
+        return Buffer<T, Dims>(Runtime::Buffer<T, Dims>::make_interleaved(width, height, channels), name);
     }
 
     static Buffer<> make_interleaved(Type t, int width, int height, int channels, const std::string &name = "") {
-        return Buffer<>(Runtime::Buffer<>::make_interleaved(t, width, height, channels),
-                        name);
+        return Buffer<>(Runtime::Buffer<>::make_interleaved(t, width, height, channels), name);
     }
 
-    static Buffer<T> make_interleaved(T *data, int width, int height, int channels, const std::string &name = "") {
-        return Buffer<T>(Runtime::Buffer<T>::make_interleaved(data, width, height, channels),
-                         name);
+    static Buffer<T, Dims> make_interleaved(T *data, int width, int height, int channels, const std::string &name = "") {
+        return Buffer<T, Dims>(Runtime::Buffer<T, Dims>::make_interleaved(data, width, height, channels), name);
     }
 
     static Buffer<Internal::add_const_if_T_is_const<T, void>>
     make_interleaved(Type t, T *data, int width, int height, int channels, const std::string &name = "") {
         using T2 = Internal::add_const_if_T_is_const<T, void>;
-        return Buffer<T2>(Runtime::Buffer<T2>::make_interleaved(t, data, width, height, channels),
-                          name);
+        return Buffer<T2, Dims>(Runtime::Buffer<T2, Dims>::make_interleaved(t, data, width, height, channels), name);
     }
 
-    template<typename T2>
-    static Buffer<T> make_with_shape_of(Buffer<T2> src,
-                                        void *(*allocate_fn)(size_t) = nullptr,
-                                        void (*deallocate_fn)(void *) = nullptr,
-                                        const std::string &name = "") {
-        return Buffer<T>(Runtime::Buffer<T>::make_with_shape_of(*src.get(), allocate_fn, deallocate_fn),
-                         name);
+    template<typename T2, int D2>
+    static Buffer<T, Dims> make_with_shape_of(Buffer<T2, D2> src,
+                                              void *(*allocate_fn)(size_t) = nullptr,
+                                              void (*deallocate_fn)(void *) = nullptr,
+                                              const std::string &name = "") {
+        return Buffer<T, Dims>(Runtime::Buffer<T, Dims>::make_with_shape_of(*src.get(), allocate_fn, deallocate_fn), name);
     }
 
-    template<typename T2>
-    static Buffer<T> make_with_shape_of(const Runtime::Buffer<T2> &src,
-                                        void *(*allocate_fn)(size_t) = nullptr,
-                                        void (*deallocate_fn)(void *) = nullptr,
-                                        const std::string &name = "") {
-        return Buffer<T>(Runtime::Buffer<T>::make_with_shape_of(src, allocate_fn, deallocate_fn),
-                         name);
+    template<typename T2, int D2>
+    static Buffer<T, Dims> make_with_shape_of(const Runtime::Buffer<T2, D2> &src,
+                                              void *(*allocate_fn)(size_t) = nullptr,
+                                              void (*deallocate_fn)(void *) = nullptr,
+                                              const std::string &name = "") {
+        return Buffer<T, Dims>(Runtime::Buffer<T, Dims>::make_with_shape_of(src, allocate_fn, deallocate_fn), name);
     }
     // @}
 
@@ -363,8 +368,8 @@ public:
     // @}
 
     /** Check if two Buffer objects point to the same underlying Buffer */
-    template<typename T2>
-    bool same_as(const Buffer<T2> &other) const {
+    template<typename T2, int D2>
+    bool same_as(const Buffer<T2, D2> &other) const {
         return (const void *)(contents.get()) == (const void *)(other.contents.get());
     }
 
@@ -377,28 +382,28 @@ public:
 
     /** Get a pointer to the underlying Runtime::Buffer */
     // @{
-    Runtime::Buffer<T> *get() {
+    Runtime::Buffer<T, Dims> *get() {
         // It's already type-checked, so no need to use as<T>.
-        return (Runtime::Buffer<T> *)(&contents->buf);
+        return (Runtime::Buffer<T, Dims> *)(&contents->buf);
     }
-    const Runtime::Buffer<T> *get() const {
-        return (const Runtime::Buffer<T> *)(&contents->buf);
+    const Runtime::Buffer<T, Dims> *get() const {
+        return (const Runtime::Buffer<T, Dims> *)(&contents->buf);
     }
     // @}
 
     // We forward numerous methods from the underlying Buffer
-#define HALIDE_BUFFER_FORWARD_CONST(method)                                                                                     \
-    template<typename... Args>                                                                                                  \
-    auto method(Args &&...args) const->decltype(std::declval<const Runtime::Buffer<T>>().method(std::forward<Args>(args)...)) { \
-        user_assert(defined()) << "Undefined buffer calling const method " #method "\n";                                        \
-        return get()->method(std::forward<Args>(args)...);                                                                      \
+#define HALIDE_BUFFER_FORWARD_CONST(method)                                                                                           \
+    template<typename... Args>                                                                                                        \
+    auto method(Args &&...args) const->decltype(std::declval<const Runtime::Buffer<T, Dims>>().method(std::forward<Args>(args)...)) { \
+        user_assert(defined()) << "Undefined buffer calling const method " #method "\n";                                              \
+        return get()->method(std::forward<Args>(args)...);                                                                            \
     }
 
-#define HALIDE_BUFFER_FORWARD(method)                                                                               \
-    template<typename... Args>                                                                                      \
-    auto method(Args &&...args)->decltype(std::declval<Runtime::Buffer<T>>().method(std::forward<Args>(args)...)) { \
-        user_assert(defined()) << "Undefined buffer calling method " #method "\n";                                  \
-        return get()->method(std::forward<Args>(args)...);                                                          \
+#define HALIDE_BUFFER_FORWARD(method)                                                                                     \
+    template<typename... Args>                                                                                            \
+    auto method(Args &&...args)->decltype(std::declval<Runtime::Buffer<T, Dims>>().method(std::forward<Args>(args)...)) { \
+        user_assert(defined()) << "Undefined buffer calling method " #method "\n";                                        \
+        return get()->method(std::forward<Args>(args)...);                                                                \
     }
 
 // This is a weird-looking but effective workaround for a deficiency in "perfect forwarding":
@@ -411,10 +416,10 @@ public:
 // and forward it as is, we can just use ... to allow an arbitrary number of commas,
 // then use __VA_ARGS__ to forward the mess as-is, and while it looks horrible, it
 // works.
-#define HALIDE_BUFFER_FORWARD_INITIALIZER_LIST(method, ...)                                            \
-    inline auto method(const __VA_ARGS__ &a)->decltype(std::declval<Runtime::Buffer<T>>().method(a)) { \
-        user_assert(defined()) << "Undefined buffer calling method " #method "\n";                     \
-        return get()->method(a);                                                                       \
+#define HALIDE_BUFFER_FORWARD_INITIALIZER_LIST(method, ...)                                                  \
+    inline auto method(const __VA_ARGS__ &a)->decltype(std::declval<Runtime::Buffer<T, Dims>>().method(a)) { \
+        user_assert(defined()) << "Undefined buffer calling method " #method "\n";                           \
+        return get()->method(a);                                                                             \
     }
 
     /** Does the same thing as the equivalent Halide::Runtime::Buffer method */
@@ -473,44 +478,50 @@ public:
 #undef HALIDE_BUFFER_FORWARD_CONST
 
     template<typename Fn, typename... Args>
-    Buffer<T> &for_each_value(Fn &&f, Args... other_buffers) {
+    Buffer<T, Dims> &for_each_value(Fn &&f, Args... other_buffers) {
         get()->for_each_value(std::forward<Fn>(f), (*std::forward<Args>(other_buffers).get())...);
         return *this;
     }
 
     template<typename Fn, typename... Args>
-    const Buffer<T> &for_each_value(Fn &&f, Args... other_buffers) const {
+    const Buffer<T, Dims> &for_each_value(Fn &&f, Args... other_buffers) const {
         get()->for_each_value(std::forward<Fn>(f), (*std::forward<Args>(other_buffers).get())...);
         return *this;
     }
 
     template<typename Fn>
-    Buffer<T> &for_each_element(Fn &&f) {
+    Buffer<T, Dims> &for_each_element(Fn &&f) {
         get()->for_each_element(std::forward<Fn>(f));
         return *this;
     }
 
     template<typename Fn>
-    const Buffer<T> &for_each_element(Fn &&f) const {
+    const Buffer<T, Dims> &for_each_element(Fn &&f) const {
         get()->for_each_element(std::forward<Fn>(f));
         return *this;
     }
 
     template<typename FnOrValue>
-    Buffer<T> &fill(FnOrValue &&f) {
+    Buffer<T, Dims> &fill(FnOrValue &&f) {
         get()->fill(std::forward<FnOrValue>(f));
         return *this;
     }
 
-    static constexpr bool has_static_halide_type = Runtime::Buffer<T>::has_static_halide_type;
+    static constexpr bool has_static_halide_type = Runtime::Buffer<T, Dims>::has_static_halide_type;
 
-    static halide_type_t static_halide_type() {
-        return Runtime::Buffer<T>::static_halide_type();
+    static constexpr halide_type_t static_halide_type() {
+        return Runtime::Buffer<T, Dims>::static_halide_type();
     }
 
-    template<typename T2>
-    static bool can_convert_from(const Buffer<T2> &other) {
-        return Halide::Runtime::Buffer<T>::can_convert_from(*other.get());
+    static constexpr bool has_static_dimensions = Runtime::Buffer<T, Dims>::has_static_dimensions;
+
+    static constexpr int static_dimensions() {
+        return Runtime::Buffer<T, Dims>::static_dimensions();
+    }
+
+    template<typename T2, int D2>
+    static bool can_convert_from(const Buffer<T2, D2> &other) {
+        return Halide::Runtime::Buffer<T, Dims>::can_convert_from(*other.get());
     }
 
     // Note that since Runtime::Buffer stores halide_type_t rather than Halide::Type,
@@ -521,43 +532,43 @@ public:
         return contents->buf.type();
     }
 
-    template<typename T2>
-    Buffer<T2> as() const {
-        return Buffer<T2>(*this);
+    template<typename T2, int D2 = Dims>
+    Buffer<T2, D2> as() const {
+        return Buffer<T2, D2>(*this);
     }
 
-    Buffer<T> copy() const {
-        return Buffer<T>(std::move(contents->buf.as<T>().copy()));
+    Buffer<T, Dims> copy() const {
+        return Buffer<T, Dims>(std::move(contents->buf.as<T, Dims>().copy()));
     }
 
-    template<typename T2>
-    void copy_from(const Buffer<T2> &other) {
+    template<typename T2, int D2>
+    void copy_from(const Buffer<T2, D2> &other) {
         contents->buf.copy_from(*other.get());
     }
 
     template<typename... Args>
-    auto operator()(int first, Args &&...args) -> decltype(std::declval<Runtime::Buffer<T>>()(first, std::forward<Args>(args)...)) {
+    auto operator()(int first, Args &&...args) -> decltype(std::declval<Runtime::Buffer<T, Dims>>()(first, std::forward<Args>(args)...)) {
         return (*get())(first, std::forward<Args>(args)...);
     }
 
     template<typename... Args>
-    auto operator()(int first, Args &&...args) const -> decltype(std::declval<const Runtime::Buffer<T>>()(first, std::forward<Args>(args)...)) {
+    auto operator()(int first, Args &&...args) const -> decltype(std::declval<const Runtime::Buffer<T, Dims>>()(first, std::forward<Args>(args)...)) {
         return (*get())(first, std::forward<Args>(args)...);
     }
 
-    auto operator()(const int *pos) -> decltype(std::declval<Runtime::Buffer<T>>()(pos)) {
+    auto operator()(const int *pos) -> decltype(std::declval<Runtime::Buffer<T, Dims>>()(pos)) {
         return (*get())(pos);
     }
 
-    auto operator()(const int *pos) const -> decltype(std::declval<const Runtime::Buffer<T>>()(pos)) {
+    auto operator()(const int *pos) const -> decltype(std::declval<const Runtime::Buffer<T, Dims>>()(pos)) {
         return (*get())(pos);
     }
 
-    auto operator()() -> decltype(std::declval<Runtime::Buffer<T>>()()) {
+    auto operator()() -> decltype(std::declval<Runtime::Buffer<T, Dims>>()()) {
         return (*get())();
     }
 
-    auto operator()() const -> decltype(std::declval<const Runtime::Buffer<T>>()()) {
+    auto operator()() const -> decltype(std::declval<const Runtime::Buffer<T, Dims>>()()) {
         return (*get())();
     }
     // @}
@@ -577,23 +588,23 @@ public:
     // @}
 
     /** Copy to the GPU, using the device API that is the default for the given Target. */
-    int copy_to_device(const Target &t = get_jit_target_from_environment()) {
-        return copy_to_device(DeviceAPI::Default_GPU, t);
+    int copy_to_device(const Target &t = get_jit_target_from_environment(), JITUserContext *context = nullptr) {
+        return copy_to_device(DeviceAPI::Default_GPU, t, context);
     }
 
     /** Copy to the GPU, using the given device API */
-    int copy_to_device(const DeviceAPI &d, const Target &t = get_jit_target_from_environment()) {
-        return contents->buf.copy_to_device(get_device_interface_for_device_api(d, t, "Buffer::copy_to_device"));
+    int copy_to_device(const DeviceAPI &d, const Target &t = get_jit_target_from_environment(), JITUserContext *context = nullptr) {
+        return contents->buf.copy_to_device(get_device_interface_for_device_api(d, t, "Buffer::copy_to_device"), context);
     }
 
     /** Allocate on the GPU, using the device API that is the default for the given Target. */
-    int device_malloc(const Target &t = get_jit_target_from_environment()) {
-        return device_malloc(DeviceAPI::Default_GPU, t);
+    int device_malloc(const Target &t = get_jit_target_from_environment(), JITUserContext *context = nullptr) {
+        return device_malloc(DeviceAPI::Default_GPU, t, context);
     }
 
     /** Allocate storage on the GPU, using the given device API */
-    int device_malloc(const DeviceAPI &d, const Target &t = get_jit_target_from_environment()) {
-        return contents->buf.device_malloc(get_device_interface_for_device_api(d, t, "Buffer::device_malloc"));
+    int device_malloc(const DeviceAPI &d, const Target &t = get_jit_target_from_environment(), JITUserContext *context = nullptr) {
+        return contents->buf.device_malloc(get_device_interface_for_device_api(d, t, "Buffer::device_malloc"), context);
     }
 
     /** Wrap a native handle, using the given device API.
@@ -601,8 +612,8 @@ public:
      * as the handle argument must match the API that the default
      * resolves to and it is clearer and more reliable to pass the
      * resolved DeviceAPI explicitly. */
-    int device_wrap_native(const DeviceAPI &d, uint64_t handle, const Target &t = get_jit_target_from_environment()) {
-        return contents->buf.device_wrap_native(get_device_interface_for_device_api(d, t, "Buffer::device_wrap_native"), handle);
+    int device_wrap_native(const DeviceAPI &d, uint64_t handle, const Target &t = get_jit_target_from_environment(), JITUserContext *context = nullptr) {
+        return contents->buf.device_wrap_native(get_device_interface_for_device_api(d, t, "Buffer::device_wrap_native"), handle, context);
     }
 };
 
