@@ -597,11 +597,12 @@ const char *const intrinsic_op_names[] = {
     "cast_mask",
     "count_leading_zeros",
     "count_trailing_zeros",
-    "declare_box_touched",
     "debug_to_file",
+    "declare_box_touched",
     "div_round_to_zero",
     "dynamic_shuffle",
     "extract_mask_element",
+    "get_user_context",
     "gpu_thread_barrier",
     "halving_add",
     "halving_sub",
@@ -616,6 +617,7 @@ const char *const intrinsic_op_names[] = {
     "lerp",
     "likely",
     "likely_if_innermost",
+    "load_typed_struct_member",
     "make_struct",
     "memoize_expr",
     "mod_round_to_zero",
