@@ -1,15 +1,12 @@
 #ifndef HALIDE_LLVM_HEADERS_H
 #define HALIDE_LLVM_HEADERS_H
 
-#if LLVM_VERSION >= 110
+#if LLVM_VERSION >= 120
 // We're good to go
 #else
-#error "Compiling Halide requires LLVM 11.0 or newer"
+#error "Compiling Halide requires LLVM 12.0 or newer"
 #endif
 
-// This seems to be required by some LLVM header, which is likely an LLVM bug.
-#include <cstddef>
-
 // No msvc warnings from llvm headers please
 #ifdef _MSC_VER
 #pragma warning(push, 0)
@@ -21,39 +18,59 @@
 #pragma clang system_header
 #endif
 
-#if WITH_WABT
+// IWYU pragma: begin_exports
+
+#if WITH_WABT || WITH_V8
 #include <lld/Common/Driver.h>
 #endif
-
-#include <llvm/ExecutionEngine/JITEventListener.h>
-#include <llvm/ExecutionEngine/MCJIT.h>
-#include <llvm/ExecutionEngine/SectionMemoryManager.h>
-
-#include "llvm/ADT/APFloat.h"
-#include "llvm/ADT/SmallVector.h"
-#include "llvm/ADT/StringRef.h"
-#include "llvm/Support/ErrorHandling.h"
+#include <llvm/ADT/APFloat.h>
+#include <llvm/ADT/ArrayRef.h>
+#include <llvm/ADT/SmallVector.h>
 #include <llvm/ADT/StringMap.h>
-#if LLVM_VERSION >= 120
+#include <llvm/ADT/StringRef.h>
+#include <llvm/ADT/Triple.h>
+#include <llvm/ADT/Twine.h>
 #include <llvm/Analysis/AliasAnalysis.h>
-#endif
 #include <llvm/Analysis/TargetLibraryInfo.h>
+#include <llvm/Analysis/TargetTransformInfo.h>
 #include <llvm/Bitcode/BitcodeReader.h>
 #include <llvm/Bitcode/BitcodeWriter.h>
 #include <llvm/ExecutionEngine/ExecutionEngine.h>
+#include <llvm/ExecutionEngine/JITEventListener.h>
+#include <llvm/ExecutionEngine/MCJIT.h>
+#include <llvm/ExecutionEngine/SectionMemoryManager.h>
+#include <llvm/IR/Constant.h>
+#include <llvm/IR/Constants.h>
+#include <llvm/IR/DataLayout.h>
+#include <llvm/IR/DerivedTypes.h>
+#include <llvm/IR/Function.h>
+#include <llvm/IR/GlobalValue.h>
+#include <llvm/IR/GlobalVariable.h>
+#include <llvm/IR/IRBuilder.h>
+#include <llvm/IR/Instructions.h>
+#include <llvm/IR/Intrinsics.h>
+#ifdef WITH_HEXAGON
+#include <llvm/IR/IntrinsicsHexagon.h>
+#endif
 #include <llvm/IR/LegacyPassManager.h>
+#include <llvm/IR/MDBuilder.h>
+#include <llvm/IR/Metadata.h>
+#include <llvm/IR/Module.h>
 #include <llvm/IR/PassTimingInfo.h>
+#include <llvm/IR/Type.h>
+#include <llvm/IR/Value.h>
 #include <llvm/IR/Verifier.h>
 #include <llvm/Linker/Linker.h>
+#include <llvm/MC/MCTargetOptions.h>
 #include <llvm/Object/ArchiveWriter.h>
 #include <llvm/Object/ObjectFile.h>
 #include <llvm/Passes/PassBuilder.h>
+#include <llvm/Support/Casting.h>
 #include <llvm/Support/CodeGen.h>
-#if LLVM_VERSION >= 120
 #include <llvm/Support/CommandLine.h>
-#endif
 #include <llvm/Support/DataExtractor.h>
 #include <llvm/Support/DynamicLibrary.h>
+#include <llvm/Support/ErrorHandling.h>
 #include <llvm/Support/FileSystem.h>
 #include <llvm/Support/FormattedStream.h>
 #if LLVM_VERSION >= 140
@@ -62,33 +79,24 @@
 #include <llvm/Support/TargetRegistry.h>
 #endif
 #include <llvm/Support/TargetSelect.h>
+#include <llvm/Support/TypeSize.h>
 #include <llvm/Support/raw_os_ostream.h>
 #include <llvm/Support/raw_ostream.h>
 #include <llvm/Target/TargetMachine.h>
+#include <llvm/Target/TargetOptions.h>
 #include <llvm/Transforms/IPO.h>
+#include <llvm/Transforms/IPO/AlwaysInliner.h>
 #include <llvm/Transforms/IPO/Inliner.h>
 #include <llvm/Transforms/IPO/PassManagerBuilder.h>
 #include <llvm/Transforms/Instrumentation.h>
 #include <llvm/Transforms/Instrumentation/AddressSanitizer.h>
+#include <llvm/Transforms/Instrumentation/SanitizerCoverage.h>
 #include <llvm/Transforms/Instrumentation/ThreadSanitizer.h>
+#include <llvm/Transforms/Scalar/GVN.h>
 #include <llvm/Transforms/Utils/ModuleUtils.h>
 #include <llvm/Transforms/Utils/SymbolRewriter.h>
 
-#include <llvm/Transforms/Scalar/GVN.h>
-
-#include <llvm/Transforms/IPO/AlwaysInliner.h>
-
-#include <llvm/Analysis/TargetTransformInfo.h>
-#include <llvm/IR/DataLayout.h>
-#include <llvm/IR/Function.h>
-#include <llvm/IR/IRBuilder.h>
-#include <llvm/IR/Intrinsics.h>
-#ifdef WITH_HEXAGON
-#include <llvm/IR/IntrinsicsHexagon.h>
-#endif
-#include <llvm/IR/MDBuilder.h>
-#include <llvm/IR/Module.h>
-#include <llvm/IR/Value.h>
+// IWYU pragma: end_exports
 
 // No msvc warnings from llvm headers please
 #ifdef _MSC_VER
@@ -119,11 +127,7 @@ inline std::string get_llvm_function_name(const llvm::Function &f) {
 }
 
 inline llvm::StructType *get_llvm_struct_type_by_name(llvm::Module *module, const char *name) {
-#if LLVM_VERSION >= 120
     return llvm::StructType::getTypeByName(module->getContext(), name);
-#else
-    return module->getTypeByName(name);
-#endif
 }
 
 }  // namespace Internal
