@@ -126,6 +126,8 @@ DECLARE_CPP_INITMOD(posix_threads)
 DECLARE_CPP_INITMOD(posix_threads_tsan)
 DECLARE_CPP_INITMOD(prefetch)
 DECLARE_CPP_INITMOD(profiler)
+DECLARE_CPP_INITMOD(timer_profiler)
+DECLARE_CPP_INITMOD(posix_timer_profiler)
 DECLARE_CPP_INITMOD(profiler_inlined)
 DECLARE_CPP_INITMOD(pseudostack)
 DECLARE_CPP_INITMOD(qurt_allocator)
@@ -228,6 +230,7 @@ DECLARE_NO_INITMOD(windows_d3d12compute_arm)
 #endif  // WITH_D3D12
 
 #ifdef WITH_X86
+DECLARE_LL_INITMOD(x86_amx)
 DECLARE_LL_INITMOD(x86_avx512)
 DECLARE_LL_INITMOD(x86_avx2)
 DECLARE_LL_INITMOD(x86_avx)
@@ -235,6 +238,7 @@ DECLARE_LL_INITMOD(x86)
 DECLARE_LL_INITMOD(x86_sse41)
 DECLARE_CPP_INITMOD(x86_cpu_features)
 #else
+DECLARE_NO_INITMOD(x86_amx)
 DECLARE_NO_INITMOD(x86_avx512)
 DECLARE_NO_INITMOD(x86_avx2)
 DECLARE_NO_INITMOD(x86_avx)
@@ -290,10 +294,21 @@ llvm::DataLayout get_data_layout_for_target(Target target) {
                 return llvm::DataLayout("e-m:o-p:32:32-p270:32:32-p271:32:32-p272:64:64-f64:32:64-f80:128-n8:16:32-S128");
             } else if (target.os == Target::IOS) {
                 return llvm::DataLayout("e-m:o-p:32:32-p270:32:32-p271:32:32-p272:64:64-f64:32:64-f80:128-n8:16:32-S128");
-            } else if (target.os == Target::Windows && !target.has_feature(Target::JIT)) {
-                return llvm::DataLayout("e-m:x-p:32:32-p270:32:32-p271:32:32-p272:64:64-i64:64-f80:32-n8:16:32-a:0:32-S32");
             } else if (target.os == Target::Windows) {
-                return llvm::DataLayout("e-m:e-p:32:32-p270:32:32-p271:32:32-p272:64:64-i64:64-f80:32-n8:16:32-a:0:32-S32");
+#if LLVM_VERSION >= 140
+                // For 32-bit MSVC targets, alignment of f80 values is 16 bytes (see https://reviews.llvm.org/D115942)
+                if (!target.has_feature(Target::JIT)) {
+                    return llvm::DataLayout("e-m:x-p:32:32-p270:32:32-p271:32:32-p272:64:64-i64:64-f80:128-n8:16:32-a:0:32-S32");
+                } else {
+                    return llvm::DataLayout("e-m:e-p:32:32-p270:32:32-p271:32:32-p272:64:64-i64:64-f80:128-n8:16:32-a:0:32-S32");
+                }
+#else
+                if (!target.has_feature(Target::JIT)) {
+                    return llvm::DataLayout("e-m:x-p:32:32-p270:32:32-p271:32:32-p272:64:64-i64:64-f80:32-n8:16:32-a:0:32-S32");
+                } else {
+                    return llvm::DataLayout("e-m:e-p:32:32-p270:32:32-p271:32:32-p272:64:64-i64:64-f80:32-n8:16:32-a:0:32-S32");
+                }
+#endif
             } else {
                 // Linux/Android
                 return llvm::DataLayout("e-m:e-p:32:32-p270:32:32-p271:32:32-p272:64:64-f64:32:64-f80:32-n8:16:32-S128");
@@ -346,11 +361,19 @@ llvm::DataLayout get_data_layout_for_target(Target target) {
             "e-m:e-p:32:32:32-a:0-n16:32-i64:64:64-i32:32:32-i16:16:16-i1:8:8"
             "-f32:32:32-f64:64:64-v32:32:32-v64:64:64-v512:512:512-v1024:1024:1024-v2048:2048:2048");
     } else if (target.arch == Target::WebAssembly) {
+#if LLVM_VERSION >= 140
+        if (target.bits == 32) {
+            return llvm::DataLayout("e-m:e-p:32:32-p10:8:8-p20:8:8-i64:64-n32:64-S128-ni:1:10:20");
+        } else {
+            return llvm::DataLayout("e-m:e-p:64:64-p10:8:8-p20:8:8-i64:64-n32:64-S128-ni:1:10:20");
+        }
+#else
         if (target.bits == 32) {
             return llvm::DataLayout("e-m:e-p:32:32-i64:64-n32:64-S128");
         } else {
             return llvm::DataLayout("e-m:e-p:64:64-i64:64-n32:64-S128");
         }
+#endif
     } else if (target.arch == Target::RISCV) {
         // TODO: Valdidate this data layout is correct for RISCV. Assumption is it is like MIPS.
         if (target.bits == 32) {
@@ -997,10 +1020,19 @@ std::unique_ptr<llvm::Module> get_initial_module_for_target(Target t, llvm::LLVM
 
             // Some environments don't support the atomics the profiler requires.
             if (t.arch != Target::MIPS && t.os != Target::NoOS && t.os != Target::QuRT) {
-                if (t.os == Target::Windows) {
-                    modules.push_back(get_initmod_windows_profiler(c, bits_64, debug));
+                if (t.has_feature(Target::ProfileByTimer)) {
+                    user_assert(!t.has_feature(Target::Profile)) << "Can only use one of Target::Profile and Target::ProfileByTimer.";
+                    // TODO(zvookin): This should work on all Posix like systems, but needs to be tested.
+                    user_assert(t.os == Target::Linux) << "The timer based profiler currently can only be used on Linux.";
+                    modules.push_back(get_initmod_profiler_inlined(c, bits_64, debug));
+                    modules.push_back(get_initmod_timer_profiler(c, bits_64, debug));
+                    modules.push_back(get_initmod_posix_timer_profiler(c, bits_64, debug));
                 } else {
-                    modules.push_back(get_initmod_profiler(c, bits_64, debug));
+                    if (t.os == Target::Windows) {
+                        modules.push_back(get_initmod_windows_profiler(c, bits_64, debug));
+                    } else {
+                        modules.push_back(get_initmod_profiler(c, bits_64, debug));
+                    }
                 }
             }
 
@@ -1056,8 +1088,20 @@ std::unique_ptr<llvm::Module> get_initial_module_for_target(Target t, llvm::LLVM
             if (t.has_feature(Target::AVX512)) {
                 modules.push_back(get_initmod_x86_avx512_ll(c));
             }
+            if (t.has_feature(Target::AVX512_SapphireRapids)) {
+                modules.push_back(get_initmod_x86_amx_ll(c));
+            }
             if (t.has_feature(Target::Profile)) {
-                user_assert(t.os != Target::WebAssemblyRuntime) << "The profiler cannot be used in a threadless environment.";
+                if (t.os == Target::WebAssemblyRuntime) {
+                    user_assert(t.has_feature(Target::WasmThreads))
+                        << "The profiler requires threads to operate; enable wasm_threads to use this under WebAssembly.";
+                }
+                modules.push_back(get_initmod_profiler_inlined(c, bits_64, debug));
+            }
+            if (t.has_feature(Target::ProfileByTimer)) {
+                user_assert(!t.has_feature(Target::Profile)) << "Can only use one of Target::Profile and Target::ProfileByTimer.";
+                // TODO(zvookin): This should work on all Posix like systems, but needs to be tested.
+                user_assert(t.os == Target::Linux) << "The timer based profiler currently can only be used on Linux.";
                 modules.push_back(get_initmod_profiler_inlined(c, bits_64, debug));
             }
             if (t.arch == Target::WebAssembly) {
