@@ -34,27 +34,27 @@ std::string output_name(const string &filename, const Module &m, const string &e
     return output_name(filename, m.name(), ext);
 }
 
-std::map<Output, std::string> single_output(const string &filename, const Module &m, Output output_type) {
+std::map<OutputFileType, std::string> single_output(const string &filename, const Module &m, OutputFileType output_type) {
     auto ext = get_output_info(m.target());
-    std::map<Output, std::string> outputs = {
+    std::map<OutputFileType, std::string> outputs = {
         {output_type, output_name(filename, m, ext.at(output_type).extension)}};
     return outputs;
 }
 
-std::map<Output, std::string> static_library_outputs(const string &filename_prefix, const Target &target) {
+std::map<OutputFileType, std::string> static_library_outputs(const string &filename_prefix, const Target &target) {
     auto ext = get_output_info(target);
-    std::map<Output, std::string> outputs = {
-        {Output::c_header, filename_prefix + ext.at(Output::c_header).extension},
-        {Output::static_library, filename_prefix + ext.at(Output::static_library).extension},
+    std::map<OutputFileType, std::string> outputs = {
+        {OutputFileType::c_header, filename_prefix + ext.at(OutputFileType::c_header).extension},
+        {OutputFileType::static_library, filename_prefix + ext.at(OutputFileType::static_library).extension},
     };
     return outputs;
 }
 
-std::map<Output, std::string> object_file_outputs(const string &filename_prefix, const Target &target) {
+std::map<OutputFileType, std::string> object_file_outputs(const string &filename_prefix, const Target &target) {
     auto ext = get_output_info(target);
-    std::map<Output, std::string> outputs = {
-        {Output::c_header, filename_prefix + ext.at(Output::c_header).extension},
-        {Output::object, filename_prefix + ext.at(Output::object).extension},
+    std::map<OutputFileType, std::string> outputs = {
+        {OutputFileType::c_header, filename_prefix + ext.at(OutputFileType::c_header).extension},
+        {OutputFileType::object, filename_prefix + ext.at(OutputFileType::object).extension},
     };
     return outputs;
 }
@@ -255,7 +255,7 @@ Func Pipeline::get_func(size_t index) {
     return Func(env.find(order[index])->second);
 }
 
-void Pipeline::compile_to(const std::map<Output, std::string> &output_files,
+void Pipeline::compile_to(const std::map<OutputFileType, std::string> &output_files,
                           const vector<Argument> &args,
                           const string &fn_name,
                           const Target &target) {
@@ -267,7 +267,7 @@ void Pipeline::compile_to_bitcode(const string &filename,
                                   const string &fn_name,
                                   const Target &target) {
     Module m = compile_to_module(args, fn_name, target);
-    m.compile(single_output(filename, m, Output::bitcode));
+    m.compile(single_output(filename, m, OutputFileType::bitcode));
 }
 
 void Pipeline::compile_to_llvm_assembly(const string &filename,
@@ -275,7 +275,7 @@ void Pipeline::compile_to_llvm_assembly(const string &filename,
                                         const string &fn_name,
                                         const Target &target) {
     Module m = compile_to_module(args, fn_name, target);
-    m.compile(single_output(filename, m, Output::llvm_assembly));
+    m.compile(single_output(filename, m, OutputFileType::llvm_assembly));
 }
 
 void Pipeline::compile_to_object(const string &filename,
@@ -284,7 +284,7 @@ void Pipeline::compile_to_object(const string &filename,
                                  const Target &target) {
     Module m = compile_to_module(args, fn_name, target);
     auto ext = get_output_info(target);
-    m.compile({{Output::object, output_name(filename, m, ext.at(Output::object).extension)}});
+    m.compile({{OutputFileType::object, output_name(filename, m, ext.at(OutputFileType::object).extension)}});
 }
 
 void Pipeline::compile_to_header(const string &filename,
@@ -292,7 +292,7 @@ void Pipeline::compile_to_header(const string &filename,
                                  const string &fn_name,
                                  const Target &target) {
     Module m = compile_to_module(args, fn_name, target);
-    m.compile(single_output(filename, m, Output::c_header));
+    m.compile(single_output(filename, m, OutputFileType::c_header));
 }
 
 void Pipeline::compile_to_assembly(const string &filename,
@@ -300,7 +300,7 @@ void Pipeline::compile_to_assembly(const string &filename,
                                    const string &fn_name,
                                    const Target &target) {
     Module m = compile_to_module(args, fn_name, target);
-    m.compile(single_output(filename, m, Output::assembly));
+    m.compile(single_output(filename, m, OutputFileType::assembly));
 }
 
 void Pipeline::compile_to_c(const string &filename,
@@ -308,7 +308,7 @@ void Pipeline::compile_to_c(const string &filename,
                             const string &fn_name,
                             const Target &target) {
     Module m = compile_to_module(args, fn_name, target);
-    m.compile(single_output(filename, m, Output::c_source));
+    m.compile(single_output(filename, m, OutputFileType::c_source));
 }
 
 void Pipeline::print_loop_nest() {
@@ -321,7 +321,7 @@ void Pipeline::compile_to_lowered_stmt(const string &filename,
                                        StmtOutputFormat fmt,
                                        const Target &target) {
     Module m = compile_to_module(args, "", target);
-    m.compile(single_output(filename, m, fmt == HTML ? Output::stmt_html : Output::stmt));
+    m.compile(single_output(filename, m, fmt == HTML ? OutputFileType::stmt_html : OutputFileType::stmt));
 }
 
 void Pipeline::compile_to_static_library(const string &filename_prefix,
@@ -359,9 +359,9 @@ void Pipeline::compile_to_file(const string &filename_prefix,
                                const Target &target) {
     Module m = compile_to_module(args, fn_name, target);
     auto ext = get_output_info(target);
-    std::map<Output, std::string> outputs = {
-        {Output::c_header, filename_prefix + ext.at(Output::c_header).extension},
-        {Output::object, filename_prefix + ext.at(Output::object).extension},
+    std::map<OutputFileType, std::string> outputs = {
+        {OutputFileType::c_header, filename_prefix + ext.at(OutputFileType::c_header).extension},
+        {OutputFileType::object, filename_prefix + ext.at(OutputFileType::object).extension},
     };
     m.compile(outputs);
 }
@@ -634,42 +634,47 @@ void Pipeline::compile_jit(const Target &target_arg) {
         }
         string file_name = program_name + "_" + name + "_" + unique_name('g').substr(1) + ".bc";
         debug(4) << "Saving bitcode to: " << file_name << "\n";
-        module.compile({{Output::bitcode, file_name}});
+        module.compile({{OutputFileType::bitcode, file_name}});
     }
 
     contents->jit_module = jit_module;
 }
 
+template<typename A, typename B>
+void set_handler(A &a, B b) {
+    a = (A)b;
+}
+
 void Pipeline::set_error_handler(void (*handler)(void *, const char *)) {
     user_assert(defined()) << "Pipeline is undefined\n";
-    contents->jit_handlers.custom_error = handler;
+    set_handler(contents->jit_handlers.custom_error, handler);
 }
 
 void Pipeline::set_custom_allocator(void *(*cust_malloc)(void *, size_t),
                                     void (*cust_free)(void *, void *)) {
     user_assert(defined()) << "Pipeline is undefined\n";
-    contents->jit_handlers.custom_malloc = cust_malloc;
-    contents->jit_handlers.custom_free = cust_free;
+    set_handler(contents->jit_handlers.custom_malloc, cust_malloc);
+    set_handler(contents->jit_handlers.custom_free, cust_free);
 }
 
 void Pipeline::set_custom_do_par_for(int (*cust_do_par_for)(void *, int (*)(void *, int, uint8_t *), int, int, uint8_t *)) {
     user_assert(defined()) << "Pipeline is undefined\n";
-    contents->jit_handlers.custom_do_par_for = cust_do_par_for;
+    set_handler(contents->jit_handlers.custom_do_par_for, cust_do_par_for);
 }
 
 void Pipeline::set_custom_do_task(int (*cust_do_task)(void *, int (*)(void *, int, uint8_t *), int, uint8_t *)) {
     user_assert(defined()) << "Pipeline is undefined\n";
-    contents->jit_handlers.custom_do_task = cust_do_task;
+    set_handler(contents->jit_handlers.custom_do_task, cust_do_task);
 }
 
 void Pipeline::set_custom_trace(int (*trace_fn)(void *, const halide_trace_event_t *)) {
     user_assert(defined()) << "Pipeline is undefined\n";
-    contents->jit_handlers.custom_trace = trace_fn;
+    set_handler(contents->jit_handlers.custom_trace, trace_fn);
 }
 
 void Pipeline::set_custom_print(void (*cust_print)(void *, const char *)) {
     user_assert(defined()) << "Pipeline is undefined\n";
-    contents->jit_handlers.custom_print = cust_print;
+    set_handler(contents->jit_handlers.custom_print, cust_print);
 }
 
 void Pipeline::set_jit_externs(const std::map<std::string, JITExtern> &externs) {
@@ -702,13 +707,20 @@ const vector<CustomLoweringPass> &Pipeline::custom_lowering_passes() {
     return contents->custom_lowering_passes;
 }
 
-const JITHandlers &Pipeline::jit_handlers() {
+JITHandlers &Pipeline::jit_handlers() {
     user_assert(defined()) << "Pipeline is undefined\n";
     return contents->jit_handlers;
 }
 
 Realization Pipeline::realize(vector<int32_t> sizes, const Target &target,
                               const ParamMap &param_map) {
+    return realize(nullptr, std::move(sizes), target, param_map);
+}
+
+Realization Pipeline::realize(JITUserContext *context,
+                              vector<int32_t> sizes,
+                              const Target &target,
+                              const ParamMap &param_map) {
     user_assert(defined()) << "Pipeline is undefined\n";
     vector<Buffer<>> bufs;
     for (auto &out : contents->outputs) {
@@ -721,13 +733,13 @@ Realization Pipeline::realize(vector<int32_t> sizes, const Target &target,
     // Do an output bounds query if we can. Otherwise just assume the
     // output size is good.
     if (!target.has_feature(Target::NoBoundsQuery)) {
-        realize(r, target, param_map);
+        realize(context, r, target, param_map);
     }
     for (size_t i = 0; i < r.size(); i++) {
         r[i].allocate();
     }
     // Do the actual computation
-    realize(r, target, param_map);
+    realize(context, r, target, param_map);
 
     // Crop back to the requested size if necessary
     bool needs_crop = false;
@@ -745,7 +757,7 @@ Realization Pipeline::realize(vector<int32_t> sizes, const Target &target,
         if (needs_crop) {
             r[i].crop(crop);
         }
-        r[i].copy_to_host();
+        r[i].copy_to_host(context);
     }
     return r;
 }
@@ -788,9 +800,8 @@ void Pipeline::trace_pipeline() {
     contents->trace_pipeline = true;
 }
 
-namespace {
-
-struct ErrorBuffer {
+namespace Internal {
+struct JITErrorBuffer {
     enum { MaxBufSize = 4096 };
     char buf[MaxBufSize];
     std::atomic<size_t> end{0};
@@ -823,39 +834,40 @@ struct ErrorBuffer {
         return std::string(buf, end);
     }
 
-    static void handler(void *ctx, const char *message) {
-        if (ctx) {
-            JITUserContext *ctx1 = (JITUserContext *)ctx;
-            ErrorBuffer *buf = (ErrorBuffer *)ctx1->user_context;
-            buf->concat(message);
+    static void handler(JITUserContext *ctx, const char *message) {
+        if (ctx && ctx->error_buffer) {
+            ctx->error_buffer->concat(message);
         }
     }
 };
 
 struct JITFuncCallContext {
-    ErrorBuffer error_buffer;
-    JITUserContext jit_context;
+    JITErrorBuffer error_buffer;
+    JITUserContext *context;
     bool custom_error_handler;
 
-    JITFuncCallContext(const JITHandlers &handlers) {
-        void *user_context = nullptr;
-        JITHandlers local_handlers = handlers;
-        if (local_handlers.custom_error == nullptr) {
-            custom_error_handler = false;
-            local_handlers.custom_error = ErrorBuffer::handler;
-            user_context = &error_buffer;
-        } else {
-            custom_error_handler = true;
+    JITFuncCallContext(JITUserContext *context, const JITHandlers &pipeline_handlers)
+        : context(context) {
+        custom_error_handler = (context->handlers.custom_error != nullptr ||
+                                pipeline_handlers.custom_error != nullptr);
+        // Hook the error handler if not set
+        if (!custom_error_handler) {
+            context->handlers.custom_error = JITErrorBuffer::handler;
         }
-        JITSharedRuntime::init_jit_user_context(jit_context, user_context, local_handlers);
 
-        debug(2) << "custom_print: " << (void *)jit_context.handlers.custom_print << "\n"
-                 << "custom_malloc: " << (void *)jit_context.handlers.custom_malloc << "\n"
-                 << "custom_free: " << (void *)jit_context.handlers.custom_free << "\n"
-                 << "custom_do_task: " << (void *)jit_context.handlers.custom_do_task << "\n"
-                 << "custom_do_par_for: " << (void *)jit_context.handlers.custom_do_par_for << "\n"
-                 << "custom_error: " << (void *)jit_context.handlers.custom_error << "\n"
-                 << "custom_trace: " << (void *)jit_context.handlers.custom_trace << "\n";
+        // Add the handlers stored in the pipeline for anything else
+        // not set, then for anything still not set, use the global
+        // active handlers.
+        JITSharedRuntime::populate_jit_handlers(context, pipeline_handlers);
+        context->error_buffer = &error_buffer;
+
+        debug(2) << "custom_print: " << (void *)context->handlers.custom_print << "\n"
+                 << "custom_malloc: " << (void *)context->handlers.custom_malloc << "\n"
+                 << "custom_free: " << (void *)context->handlers.custom_free << "\n"
+                 << "custom_do_task: " << (void *)context->handlers.custom_do_task << "\n"
+                 << "custom_do_par_for: " << (void *)context->handlers.custom_do_par_for << "\n"
+                 << "custom_error: " << (void *)context->handlers.custom_error << "\n"
+                 << "custom_trace: " << (void *)context->handlers.custom_trace << "\n";
     }
 
     void report_if_error(int exit_status) {
@@ -876,8 +888,7 @@ struct JITFuncCallContext {
         report_if_error(exit_status);
     }
 };
-
-}  // namespace
+}  // namespace Internal
 
 struct Pipeline::JITCallArgs {
     size_t size{0};
@@ -914,7 +925,7 @@ public:
 // currently bound value for all of the params and image
 // params.
 void Pipeline::prepare_jit_call_arguments(RealizationArg &outputs, const Target &target,
-                                          const ParamMap &param_map, void *user_context,
+                                          const ParamMap &param_map, JITUserContext **user_context,
                                           bool is_bounds_inference, JITCallArgs &args_result) {
     user_assert(defined()) << "Can't realize an undefined Pipeline\n";
 
@@ -1048,7 +1059,15 @@ int Pipeline::call_jit_code(const Target &target, const JITCallArgs &args) {
     return contents->jit_module.argv_function()(args.store);
 }
 
-void Pipeline::realize(RealizationArg outputs, const Target &t,
+void Pipeline::realize(RealizationArg outputs,
+                       const Target &t,
+                       const ParamMap &param_map) {
+    realize(nullptr, std::move(outputs), t, param_map);
+}
+
+void Pipeline::realize(JITUserContext *context,
+                       RealizationArg outputs,
+                       const Target &t,
                        const ParamMap &param_map) {
     Target target = t;
     user_assert(defined()) << "Can't realize an undefined Pipeline\n";
@@ -1090,12 +1109,15 @@ void Pipeline::realize(RealizationArg outputs, const Target &t,
     compile_jit(target);
 
     // This has to happen after a runtime has been compiled in compile_jit.
-    JITFuncCallContext jit_context(jit_handlers());
-    void *user_context_storage = &jit_context.jit_context;
+    JITUserContext empty_jit_user_context{};
+    if (!context) {
+        context = &empty_jit_user_context;
+    }
+    JITFuncCallContext jit_call_context(context, jit_handlers());
 
     JITCallArgs args(contents->inferred_args.size() + outputs.size());
     prepare_jit_call_arguments(outputs, target, param_map,
-                               &user_context_storage, false, args);
+                               &context, false, args);
 
     // The handlers in the jit_context default to the default handlers
     // in the runtime of the shared module (e.g. halide_print_impl,
@@ -1139,36 +1161,45 @@ void Pipeline::realize(RealizationArg outputs, const Target &t,
     debug(2) << "Back from jitted function. Exit status was " << exit_status << "\n";
 
     // If we're profiling, report runtimes and reset profiler stats.
-    if (target.has_feature(Target::Profile)) {
+    if (target.has_feature(Target::Profile) || target.has_feature(Target::ProfileByTimer)) {
         JITModule::Symbol report_sym =
             contents->jit_module.find_symbol_by_name("halide_profiler_report");
         JITModule::Symbol reset_sym =
             contents->jit_module.find_symbol_by_name("halide_profiler_reset");
         if (report_sym.address && reset_sym.address) {
-            void *uc = &jit_context.jit_context;
-            void (*report_fn_ptr)(void *) = (void (*)(void *))(report_sym.address);
-            report_fn_ptr(uc);
+            void (*report_fn_ptr)(JITUserContext *) = (void (*)(JITUserContext *))(report_sym.address);
+            report_fn_ptr(context);
 
             void (*reset_fn_ptr)() = (void (*)())(reset_sym.address);
             reset_fn_ptr();
         }
     }
 
-    jit_context.finalize(exit_status);
+    jit_call_context.finalize(exit_status);
 }
 
 void Pipeline::infer_input_bounds(RealizationArg outputs, const Target &target, const ParamMap &param_map) {
+    infer_input_bounds(nullptr, std::move(outputs), target, param_map);
+}
+
+void Pipeline::infer_input_bounds(JITUserContext *context,
+                                  RealizationArg outputs,
+                                  const Target &target,
+                                  const ParamMap &param_map) {
     user_assert(!target.has_feature(Target::NoBoundsQuery)) << "You may not call infer_input_bounds() with Target::NoBoundsQuery set.";
     compile_jit(target);
 
     // This has to happen after a runtime has been compiled in compile_jit.
-    JITFuncCallContext jit_context(jit_handlers());
-    void *user_context_storage = &jit_context.jit_context;
+    JITUserContext empty_user_context = {};
+    if (!context) {
+        context = &empty_user_context;
+    }
+    JITFuncCallContext jit_context(context, jit_handlers());
 
     size_t args_size = contents->inferred_args.size() + outputs.size();
     JITCallArgs args(args_size);
     prepare_jit_call_arguments(outputs, contents->jit_target, param_map,
-                               &user_context_storage, true, args);
+                               &context, true, args);
 
     struct TrackedBuffer {
         // The query buffer, and a backup to check for changes. We
@@ -1265,13 +1296,20 @@ void Pipeline::infer_input_bounds(RealizationArg outputs, const Target &target,
 void Pipeline::infer_input_bounds(const std::vector<int32_t> &sizes,
                                   const Target &target,
                                   const ParamMap &param_map) {
+    infer_input_bounds(nullptr, sizes, target, param_map);
+}
+
+void Pipeline::infer_input_bounds(JITUserContext *context,
+                                  const std::vector<int32_t> &sizes,
+                                  const Target &target,
+                                  const ParamMap &param_map) {
     user_assert(defined()) << "Can't infer input bounds on an undefined Pipeline.\n";
     vector<Buffer<>> bufs;
     for (Type t : contents->outputs[0].output_types()) {
         bufs.emplace_back(t, sizes);
     }
     Realization r(bufs);
-    infer_input_bounds(r, target, param_map);
+    infer_input_bounds(context, r, target, param_map);
 }
 
 void Pipeline::invalidate_cache() {
