@@ -49,8 +49,8 @@ class FindImageInputs : public IRVisitor {
                 }
             }
         }
-        for (size_t i = 0; i < call->args.size(); i++) {
-            call->args[i].accept(this);
+        for (const auto &arg : call->args) {
+            arg.accept(this);
         }
     }
 
@@ -152,11 +152,13 @@ class ExprCost : public IRVisitor {
 
     void visit(const Call *call) override {
         if (call->is_intrinsic(Call::if_then_else)) {
-            internal_assert(call->args.size() == 3);
+            internal_assert(call->args.size() == 2 || call->args.size() == 3);
 
             int64_t current_arith = arith, current_memory = memory;
             arith = 0, memory = 0;
-            call->args[2].accept(this);
+            if (call->args.size() == 3) {
+                call->args[2].accept(this);
+            }
 
             // Check if this if_then_else is because of tracing or print_when.
             // If it is, we should only take into account the cost of computing
@@ -238,8 +240,8 @@ class ExprCost : public IRVisitor {
             }
         }
 
-        for (size_t i = 0; i < call->args.size(); i++) {
-            call->args[i].accept(this);
+        for (const auto &arg : call->args) {
+            arg.accept(this);
         }
     }
 
@@ -316,8 +318,8 @@ Expr get_func_value_size(const Function &f) {
     Expr size = 0;
     const vector<Type> &types = f.output_types();
     internal_assert(!types.empty());
-    for (size_t i = 0; i < types.size(); i++) {
-        size += types[i].bytes();
+    for (auto type : types) {
+        size += type.bytes();
     }
     return simplify(size);
 }
