@@ -173,8 +173,6 @@ Target calculate_host_target() {
             if ((info2[1] & avx512_cannonlake) == avx512_cannonlake) {
                 initial_features.push_back(Target::AVX512_Cannonlake);
 
-#if LLVM_VERSION >= 120
-                // Sapphire Rapids support was added in LLVM 12, so earlier versions cannot support this CPU's features.
                 const uint32_t avx512vnni = 1U << 11;  // vnni result in ecx
                 const uint32_t avx512bf16 = 1U << 5;   // bf16 result in eax, with cpuid(eax=7, ecx=1)
                 int info3[4];
@@ -183,7 +181,6 @@ Target calculate_host_target() {
                     (info3[0] & avx512bf16) == avx512bf16) {
                     initial_features.push_back(Target::AVX512_SapphireRapids);
                 }
-#endif
             }
         }
     }
@@ -257,8 +254,10 @@ Target::Feature calculate_host_cuda_capability(Target t) {
         return Target::CUDACapability70;
     } else if (ver < 80) {
         return Target::CUDACapability75;
-    } else {
+    } else if (ver < 86 || LLVM_VERSION < 130) {
         return Target::CUDACapability80;
+    } else {
+        return Target::CUDACapability86;
     }
 }
 
@@ -332,6 +331,7 @@ const std::map<std::string, Target::Feature> feature_name_map = {
     {"cuda_capability_70", Target::CUDACapability70},
     {"cuda_capability_75", Target::CUDACapability75},
     {"cuda_capability_80", Target::CUDACapability80},
+    {"cuda_capability_86", Target::CUDACapability86},
     {"opencl", Target::OpenCL},
     {"cl_doubles", Target::CLDoubles},
     {"cl_half", Target::CLHalf},
@@ -384,6 +384,8 @@ const std::map<std::string, Target::Feature> feature_name_map = {
     {"llvm_large_code_model", Target::LLVMLargeCodeModel},
     {"rvv", Target::RVV},
     {"armv81a", Target::ARMv81a},
+    {"sanitizer_coverage", Target::SanitizerCoverage},
+    {"profile_by_timer", Target::ProfileByTimer},
     // NOTE: When adding features to this map, be sure to update PyEnums.cpp as well.
 };
 
@@ -420,6 +422,9 @@ Target get_jit_target_from_environment() {
 #if __has_feature(thread_sanitizer)
     host.set_feature(Target::TSAN);
 #endif
+#if __has_feature(coverage_sanitizer)
+    host.set_feature(Target::SanitizerCoverage);
+#endif
 #endif
     string target = Internal::get_env_variable("HL_JIT_TARGET");
     if (target.empty()) {
@@ -431,6 +436,8 @@ Target get_jit_target_from_environment() {
             << "HL_JIT_TARGET must match the host OS, architecture, and bit width.\n"
             << "HL_JIT_TARGET was " << target << ". "
             << "Host is " << host.to_string() << ".\n";
+        user_assert(!t.has_feature(Target::NoBoundsQuery))
+            << "The Halide JIT requires the use of bounds query, but HL_JIT_TARGET was specified with no_bounds_query: " << target;
         return t;
     }
 }
@@ -441,7 +448,7 @@ bool merge_string(Target &t, const std::string &target) {
     vector<string> tokens;
     size_t first_dash;
     while ((first_dash = rest.find('-')) != string::npos) {
-        //Internal::debug(0) << first_dash << ", " << rest << "\n";
+        // Internal::debug(0) << first_dash << ", " << rest << "\n";
         tokens.push_back(rest.substr(0, first_dash));
         rest = rest.substr(first_dash + 1);
     }
@@ -497,7 +504,8 @@ bool merge_string(Target &t, const std::string &target) {
         !t.has_feature(Target::CUDACapability61) &&
         !t.has_feature(Target::CUDACapability70) &&
         !t.has_feature(Target::CUDACapability75) &&
-        !t.has_feature(Target::CUDACapability80)) {
+        !t.has_feature(Target::CUDACapability80) &&
+        !t.has_feature(Target::CUDACapability86)) {
         // Detect host cuda capability
         t.set_feature(get_host_cuda_capability(t));
     }
@@ -770,6 +778,9 @@ int Target::get_cuda_capability_lower_bound() const {
     if (has_feature(Target::CUDACapability80)) {
         return 80;
     }
+    if (has_feature(Target::CUDACapability86)) {
+        return 86;
+    }
     return 20;
 }
 
@@ -961,6 +972,7 @@ bool Target::get_runtime_compatible_target(const Target &other, Target &result)
         CUDACapability70,
         CUDACapability75,
         CUDACapability80,
+        CUDACapability86,
         HVX_v62,
         HVX_v65,
         HVX_v66,
@@ -987,7 +999,7 @@ bool Target::get_runtime_compatible_target(const Target &other, Target &result)
     // clang-format on
 
     // clang-format off
-    const std::array<Feature, 12> matching_features = {{
+    const std::array<Feature, 10> matching_features = {{
         ASAN,
         Debug,
         HexagonDma,
@@ -997,6 +1009,7 @@ bool Target::get_runtime_compatible_target(const Target &other, Target &result)
         SoftFloatABI,
         TSAN,
         WasmThreads,
+        SanitizerCoverage,
     }};
     // clang-format on
 
@@ -1025,7 +1038,7 @@ bool Target::get_runtime_compatible_target(const Target &other, Target &result)
     }
 
     if ((features & matching_mask) != (other.features & matching_mask)) {
-        Internal::debug(1) << "runtime targets must agree on SoftFloatABI, Debug, TSAN, ASAN, MSAN, HVX, HexagonDma, and HVX_shared_object\n"
+        Internal::debug(1) << "runtime targets must agree on SoftFloatABI, Debug, TSAN, ASAN, MSAN, HVX, HexagonDma, HVX_shared_object, SanitizerCoverage\n"
                            << "  this:  " << *this << "\n"
                            << "  other: " << other << "\n";
         return false;
@@ -1037,11 +1050,6 @@ bool Target::get_runtime_compatible_target(const Target &other, Target &result)
     Target output = Target{os, arch, bits};
     output.features = ((features | other.features) & union_mask) | ((features | other.features) & matching_mask) | ((features & other.features) & intersection_mask);
 
-#if LLVM_VERSION < 120
-    // We require LLVM 12+ to compile SapphireRapids features.
-    output.features.reset(AVX512_SapphireRapids);
-#endif
-
     // Pick tight lower bound for CUDA capability. Use fall-through to clear redundant features
     int cuda_a = get_cuda_capability_lower_bound();
     int cuda_b = other.get_cuda_capability_lower_bound();
@@ -1074,6 +1082,9 @@ bool Target::get_runtime_compatible_target(const Target &other, Target &result)
     if (cuda_capability < 80) {
         output.features.reset(CUDACapability80);
     }
+    if (cuda_capability < 86) {
+        output.features.reset(CUDACapability86);
+    }
 
     // Pick tight lower bound for HVX version. Use fall-through to clear redundant features
     int hvx_a = get_hvx_lower_bound(*this);
