@@ -1041,7 +1041,6 @@ const Bound &LoopNest::get_bounds(const FunctionDAG::Node *f) const {
 
     // Compute the region required
     if (f->is_output && is_root()) {
-        internal_assert(f->outgoing_edges.empty()) << "Outputs that access other outputs not yet supported\n";
         // It's an output. Use the bounds estimate.
         for (int i = 0; i < f->dimensions; i++) {
             bound->region_required(i) = f->estimated_region_required[i];
@@ -1245,12 +1244,12 @@ bool LoopNest::computes(const FunctionDAG::Node *f) const {
 // Inline a Func into all consumers within this loop.
 void LoopNest::inline_func(const FunctionDAG::Node *f) {
     // Inline it into the children
-    for (size_t i = 0; i < children.size(); i++) {
-        if (children[i]->calls(f)) {
+    for (auto &child : children) {
+        if (child->calls(f)) {
             std::unique_ptr<LoopNest> new_child{new LoopNest};
-            new_child->copy_from(*children[i]);
+            new_child->copy_from(*child);
             new_child->inline_func(f);
-            children[i] = new_child.release();
+            child = new_child.release();
         }
     }
 
@@ -1436,6 +1435,7 @@ vector<IntrusivePtr<const LoopNest>> LoopNest::compute_in_tiles(const FunctionDA
     vector<IntrusivePtr<const LoopNest>> result;
 
     // Some pruning to not waste time on terrible states
+    bool must_tile_to_vectorize = false;
     if (parent) {
         const auto &bounds_here = get_bounds(f);
         const auto &bounds_at_parent = parent->get_bounds(f);
@@ -1447,7 +1447,7 @@ vector<IntrusivePtr<const LoopNest>> LoopNest::compute_in_tiles(const FunctionDA
         int64_t e = p.extent();
         int64_t ep = p_parent.extent();
         if (ep >= f->vector_size && e < f->vector_size) {
-            return result;
+            must_tile_to_vectorize = true;
         }
 
         // Don't descend into loops if the bounds required don't
@@ -1477,7 +1477,8 @@ vector<IntrusivePtr<const LoopNest>> LoopNest::compute_in_tiles(const FunctionDA
     }
 
     // Place the computation directly inside this loop (provided it's not a SIMD loop)
-    if (!innermost &&
+    if (!must_tile_to_vectorize &&
+        !innermost &&
         (!in_realization ||
          size.empty() ||
          vector_dim == -1 ||
@@ -1624,8 +1625,7 @@ vector<IntrusivePtr<const LoopNest>> LoopNest::compute_in_tiles(const FunctionDA
 
         const auto &c = children[child];
         int num_ones = 0;
-        for (size_t i = 0; i < c->size.size(); i++) {
-            int64_t s = c->size[i];
+        for (int64_t s : c->size) {
             num_ones += (s == 1) ? 1 : 0;
         }
 
