@@ -72,7 +72,7 @@ ALWAYS_INLINE T get_cuda_symbol(void *user_context, const char *name, bool optio
 // Load a CUDA shared object/dll and get the CUDA API function pointers from it.
 WEAK void load_libcuda(void *user_context) {
     debug(user_context) << "    load_libcuda (user_context: " << user_context << ")\n";
-    halide_assert(user_context, cuInit == nullptr);
+    halide_abort_if_false(user_context, cuInit == nullptr);
 
 // clang-format off
 #define CUDA_FN(ret, fn, args)               fn = get_cuda_symbol<ret(CUDAAPI *) args>(user_context, #fn);        // NOLINT(bugprone-macro-parentheses)
@@ -136,13 +136,13 @@ extern "C" {
 // - A call to halide_cuda_acquire_context is followed by a matching call to
 //   halide_cuda_release_context. halide_cuda_acquire_context should block while a
 //   previous call (if any) has not yet been released via halide_cuda_release_context.
-WEAK int halide_cuda_acquire_context(void *user_context, CUcontext *ctx, bool create = true) {
+WEAK int halide_default_cuda_acquire_context(void *user_context, CUcontext *ctx, bool create = true) {
     // TODO: Should we use a more "assertive" assert? these asserts do
     // not block execution on failure.
-    halide_assert(user_context, ctx != nullptr);
+    halide_abort_if_false(user_context, ctx != nullptr);
 
     // If the context has not been initialized, initialize it now.
-    halide_assert(user_context, &context != nullptr);
+    halide_abort_if_false(user_context, &context != nullptr);
 
     // Note that this null-check of the context is *not* locked with
     // respect to device_release, so we may get a non-null context
@@ -179,7 +179,7 @@ WEAK int halide_cuda_acquire_context(void *user_context, CUcontext *ctx, bool cr
     return 0;
 }
 
-WEAK int halide_cuda_release_context(void *user_context) {
+WEAK int halide_default_cuda_release_context(void *user_context) {
     return 0;
 }
 
@@ -188,7 +188,7 @@ WEAK int halide_cuda_release_context(void *user_context) {
 // for the context (nullptr stream). The context is passed in for convenience, but
 // any sort of scoping must be handled by that of the
 // halide_cuda_acquire_context/halide_cuda_release_context pair, not this call.
-WEAK int halide_cuda_get_stream(void *user_context, CUcontext ctx, CUstream *stream) {
+WEAK int halide_default_cuda_get_stream(void *user_context, CUcontext ctx, CUstream *stream) {
     // There are two default streams we could use. stream 0 is fully
     // synchronous. stream 2 gives a separate non-blocking stream per
     // thread.
@@ -198,6 +198,53 @@ WEAK int halide_cuda_get_stream(void *user_context, CUcontext ctx, CUstream *str
 
 }  // extern "C"
 
+namespace Halide {
+namespace Runtime {
+namespace Internal {
+namespace CUDA {
+
+WEAK halide_cuda_acquire_context_t acquire_context = (halide_cuda_acquire_context_t)halide_default_cuda_acquire_context;
+WEAK halide_cuda_release_context_t release_context = (halide_cuda_release_context_t)halide_default_cuda_release_context;
+WEAK halide_cuda_get_stream_t get_stream = (halide_cuda_get_stream_t)halide_default_cuda_get_stream;
+
+}  // namespace CUDA
+}  // namespace Internal
+}  // namespace Runtime
+}  // namespace Halide
+
+extern "C" {
+
+WEAK int halide_cuda_acquire_context(void *user_context, CUcontext *ctx, bool create = true) {
+    return CUDA::acquire_context(user_context, (void **)ctx, create);
+}
+
+WEAK halide_cuda_acquire_context_t halide_set_cuda_acquire_context(halide_cuda_acquire_context_t handler) {
+    halide_cuda_acquire_context_t result = CUDA::acquire_context;
+    CUDA::acquire_context = handler;
+    return result;
+}
+
+WEAK int halide_cuda_release_context(void *user_context) {
+    return CUDA::release_context(user_context);
+}
+
+WEAK halide_cuda_release_context_t halide_set_cuda_release_context(halide_cuda_release_context_t handler) {
+    halide_cuda_release_context_t result = CUDA::release_context;
+    CUDA::release_context = handler;
+    return result;
+}
+
+WEAK int halide_cuda_get_stream(void *user_context, CUcontext ctx, CUstream *stream) {
+    return CUDA::get_stream(user_context, (void *)ctx, (void **)stream);
+}
+
+WEAK halide_cuda_get_stream_t halide_set_cuda_get_stream(halide_cuda_get_stream_t handler) {
+    halide_cuda_get_stream_t result = CUDA::get_stream;
+    CUDA::get_stream = handler;
+    return result;
+}
+}
+
 namespace Halide {
 namespace Runtime {
 namespace Internal {
@@ -229,8 +276,8 @@ public:
         // overridden, we may still need to load libcuda
         ensure_libcuda_init(user_context);
 
-        halide_assert(user_context, context != nullptr);
-        halide_assert(user_context, cuInit != nullptr);
+        halide_abort_if_false(user_context, context != nullptr);
+        halide_abort_if_false(user_context, cuInit != nullptr);
 
         error = cuCtxPushCurrent(context);
     }
@@ -395,6 +442,9 @@ WEAK CUresult create_cuda_context(void *user_context, CUcontext *ctx) {
         case 7:
             threads_per_core = 64;
             break;
+        case 8:
+            threads_per_core = 128;
+            break;
         default:
             threads_per_core = 0;
             break;
@@ -528,7 +578,7 @@ WEAK int halide_cuda_initialize_kernels(void *user_context, void **state_ptr, co
                                               compile_kernel, user_context, ptx_src, size)) {
         return halide_error_code_generic_error;
     }
-    halide_assert(user_context, loaded_module != nullptr);
+    halide_abort_if_false(user_context, loaded_module != nullptr);
 
 #ifdef DEBUG_RUNTIME
     uint64_t t_after = halide_current_time_ns(user_context);
@@ -611,7 +661,7 @@ WEAK int halide_cuda_device_free(void *user_context, halide_buffer_t *buf) {
     uint64_t t_before = halide_current_time_ns(user_context);
 #endif
 
-    halide_assert(user_context, validate_device_pointer(user_context, buf));
+    halide_abort_if_false(user_context, validate_device_pointer(user_context, buf));
 
     CUresult err = CUDA_SUCCESS;
     if (halide_can_reuse_device_allocations(user_context)) {
@@ -685,7 +735,7 @@ WEAK int halide_cuda_device_release(void *user_context) {
         if (err != CUDA_SUCCESS) {
             err = cuCtxSynchronize();
         }
-        halide_assert(user_context, err == CUDA_SUCCESS || err == CUDA_ERROR_DEINITIALIZED);
+        halide_abort_if_false(user_context, err == CUDA_SUCCESS || err == CUDA_ERROR_DEINITIALIZED);
 
         // Dump the contents of the free list, ignoring errors.
         halide_cuda_release_unused_device_allocations(user_context);
@@ -704,7 +754,7 @@ WEAK int halide_cuda_device_release(void *user_context) {
                 debug(user_context) << "    cuCtxDestroy " << context << "\n";
                 err = cuProfilerStop();
                 err = cuCtxDestroy(context);
-                halide_assert(user_context, err == CUDA_SUCCESS || err == CUDA_ERROR_DEINITIALIZED);
+                halide_abort_if_false(user_context, err == CUDA_SUCCESS || err == CUDA_ERROR_DEINITIALIZED);
                 context = nullptr;
             }
         }  // spinlock
@@ -729,16 +779,16 @@ WEAK int halide_cuda_device_malloc(void *user_context, halide_buffer_t *buf) {
     if (halide_can_reuse_device_allocations(user_context)) {
         size = quantize_allocation_size(size);
     }
-    halide_assert(user_context, size != 0);
+    halide_abort_if_false(user_context, size != 0);
     if (buf->device) {
         // This buffer already has a device allocation
-        halide_assert(user_context, validate_device_pointer(user_context, buf, size));
+        halide_abort_if_false(user_context, validate_device_pointer(user_context, buf, size));
         return 0;
     }
 
     // Check all strides positive.
     for (int i = 0; i < buf->dimensions; i++) {
-        halide_assert(user_context, buf->dim[i].stride >= 0);
+        halide_abort_if_false(user_context, buf->dim[i].stride >= 0);
     }
 
     debug(user_context) << "    allocating " << *buf << "\n";
@@ -827,7 +877,7 @@ WEAK int halide_cuda_device_malloc(void *user_context, halide_buffer_t *buf) {
             debug(user_context) << (void *)p << "\n";
         }
     }
-    halide_assert(user_context, p);
+    halide_abort_if_false(user_context, p);
     buf->device = p;
     buf->device_interface = &cuda_device_interface;
     buf->device_interface->impl->use_module();
@@ -842,7 +892,8 @@ WEAK int halide_cuda_device_malloc(void *user_context, halide_buffer_t *buf) {
 
 namespace {
 WEAK int cuda_do_multidimensional_copy(void *user_context, const device_copy &c,
-                                       uint64_t src, uint64_t dst, int d, bool from_host, bool to_host) {
+                                       uint64_t src, uint64_t dst, int d, bool from_host, bool to_host,
+                                       CUstream stream) {
     if (d > MAX_COPY_DIMS) {
         error(user_context) << "Buffer has too many dimensions to copy to/from GPU\n";
         return -1;
@@ -855,15 +906,27 @@ WEAK int cuda_do_multidimensional_copy(void *user_context, const device_copy &c,
         if (!from_host && to_host) {
             debug(user_context) << "cuMemcpyDtoH(" << (void *)dst << ", " << (void *)src << ", " << c.chunk_size << ")\n";
             copy_name = "cuMemcpyDtoH";
-            err = cuMemcpyDtoH((void *)dst, (CUdeviceptr)src, c.chunk_size);
+            if (stream) {
+                err = cuMemcpyDtoHAsync((void *)dst, (CUdeviceptr)src, c.chunk_size, stream);
+            } else {
+                err = cuMemcpyDtoH((void *)dst, (CUdeviceptr)src, c.chunk_size);
+            }
         } else if (from_host && !to_host) {
             debug(user_context) << "cuMemcpyHtoD(" << (void *)dst << ", " << (void *)src << ", " << c.chunk_size << ")\n";
             copy_name = "cuMemcpyHtoD";
-            err = cuMemcpyHtoD((CUdeviceptr)dst, (void *)src, c.chunk_size);
+            if (stream) {
+                err = cuMemcpyHtoDAsync((CUdeviceptr)dst, (void *)src, c.chunk_size, stream);
+            } else {
+                err = cuMemcpyHtoD((CUdeviceptr)dst, (void *)src, c.chunk_size);
+            }
         } else if (!from_host && !to_host) {
             debug(user_context) << "cuMemcpyDtoD(" << (void *)dst << ", " << (void *)src << ", " << c.chunk_size << ")\n";
             copy_name = "cuMemcpyDtoD";
-            err = cuMemcpyDtoD((CUdeviceptr)dst, (CUdeviceptr)src, c.chunk_size);
+            if (stream) {
+                err = cuMemcpyDtoDAsync((CUdeviceptr)dst, (CUdeviceptr)src, c.chunk_size, stream);
+            } else {
+                err = cuMemcpyDtoD((CUdeviceptr)dst, (CUdeviceptr)src, c.chunk_size);
+            }
         } else if (dst != src) {
             debug(user_context) << "memcpy(" << (void *)dst << ", " << (void *)src << ", " << c.chunk_size << ")\n";
             // Could reach here if a user called directly into the
@@ -878,7 +941,7 @@ WEAK int cuda_do_multidimensional_copy(void *user_context, const device_copy &c,
     } else {
         ssize_t src_off = 0, dst_off = 0;
         for (int i = 0; i < (int)c.extent[d - 1]; i++) {
-            int err = cuda_do_multidimensional_copy(user_context, c, src + src_off, dst + dst_off, d - 1, from_host, to_host);
+            int err = cuda_do_multidimensional_copy(user_context, c, src + src_off, dst + dst_off, d - 1, from_host, to_host, stream);
             dst_off += c.dst_stride_bytes[d - 1];
             src_off += c.src_stride_bytes[d - 1];
             if (err) {
@@ -894,12 +957,12 @@ WEAK int halide_cuda_buffer_copy(void *user_context, struct halide_buffer_t *src
                                  const struct halide_device_interface_t *dst_device_interface,
                                  struct halide_buffer_t *dst) {
     // We only handle copies to cuda or to host
-    halide_assert(user_context, dst_device_interface == nullptr ||
-                                    dst_device_interface == &cuda_device_interface);
+    halide_abort_if_false(user_context, dst_device_interface == nullptr ||
+                                            dst_device_interface == &cuda_device_interface);
 
     if ((src->device_dirty() || src->host == nullptr) &&
         src->device_interface != &cuda_device_interface) {
-        halide_assert(user_context, dst_device_interface == &cuda_device_interface);
+        halide_abort_if_false(user_context, dst_device_interface == &cuda_device_interface);
         // This is handled at the higher level.
         return halide_error_code_incompatible_device_interface;
     }
@@ -909,8 +972,8 @@ WEAK int halide_cuda_buffer_copy(void *user_context, struct halide_buffer_t *src
                      (src->host_dirty() && src->host != nullptr);
     bool to_host = !dst_device_interface;
 
-    halide_assert(user_context, from_host || src->device);
-    halide_assert(user_context, to_host || dst->device);
+    halide_abort_if_false(user_context, from_host || src->device);
+    halide_abort_if_false(user_context, to_host || dst->device);
 
     device_copy c = make_buffer_copy(src, from_host, dst, to_host);
 
@@ -928,14 +991,22 @@ WEAK int halide_cuda_buffer_copy(void *user_context, struct halide_buffer_t *src
 #ifdef DEBUG_RUNTIME
         uint64_t t_before = halide_current_time_ns(user_context);
         if (!from_host) {
-            halide_assert(user_context, validate_device_pointer(user_context, src));
+            halide_abort_if_false(user_context, validate_device_pointer(user_context, src));
         }
         if (!to_host) {
-            halide_assert(user_context, validate_device_pointer(user_context, dst));
+            halide_abort_if_false(user_context, validate_device_pointer(user_context, dst));
         }
 #endif
 
-        err = cuda_do_multidimensional_copy(user_context, c, c.src + c.src_begin, c.dst, dst->dimensions, from_host, to_host);
+        CUstream stream = nullptr;
+        if (cuStreamSynchronize != nullptr) {
+            int result = halide_cuda_get_stream(user_context, ctx.context, &stream);
+            if (result != 0) {
+                error(user_context) << "CUDA: In cuda_do_multidimensional_copy, halide_cuda_get_stream returned " << result << "\n";
+            }
+        }
+
+        err = cuda_do_multidimensional_copy(user_context, c, c.src + c.src_begin, c.dst, dst->dimensions, from_host, to_host, stream);
 
 #ifdef DEBUG_RUNTIME
         uint64_t t_after = halide_current_time_ns(user_context);
@@ -1068,7 +1139,7 @@ WEAK int halide_cuda_run(void *user_context,
 
     CUmodule mod{};
     bool found = compilation_cache.lookup(ctx.context, state_ptr, mod);
-    halide_assert(user_context, found && mod != nullptr);
+    halide_abort_if_false(user_context, found && mod != nullptr);
 
     debug(user_context) << "Got module " << mod << "\n";
     CUfunction f;
@@ -1095,7 +1166,7 @@ WEAK int halide_cuda_run(void *user_context,
     uint64_t *dev_handles = (uint64_t *)malloc(num_args * sizeof(uint64_t));
     for (size_t i = 0; i <= num_args; i++) {  // Get nullptr at end.
         if (arg_is_buffer[i]) {
-            halide_assert(user_context, arg_sizes[i] == sizeof(uint64_t));
+            halide_abort_if_false(user_context, arg_sizes[i] == sizeof(uint64_t));
             dev_handles[i] = ((halide_buffer_t *)args[i])->device;
             translated_args[i] = &(dev_handles[i]);
             debug(user_context) << "    halide_cuda_run translated arg" << (int)i
@@ -1155,7 +1226,7 @@ WEAK int halide_cuda_device_and_host_free(void *user_context, struct halide_buff
 }
 
 WEAK int halide_cuda_wrap_device_ptr(void *user_context, struct halide_buffer_t *buf, uint64_t device_ptr) {
-    halide_assert(user_context, buf->device == 0);
+    halide_abort_if_false(user_context, buf->device == 0);
     if (buf->device != 0) {
         return -2;
     }
@@ -1177,7 +1248,7 @@ WEAK int halide_cuda_detach_device_ptr(void *user_context, struct halide_buffer_
     if (buf->device == 0) {
         return 0;
     }
-    halide_assert(user_context, buf->device_interface == &cuda_device_interface);
+    halide_abort_if_false(user_context, buf->device_interface == &cuda_device_interface);
     buf->device_interface->impl->release_module();
     buf->device = 0;
     buf->device_interface = nullptr;
@@ -1188,7 +1259,7 @@ WEAK uintptr_t halide_cuda_get_device_ptr(void *user_context, struct halide_buff
     if (buf->device == 0) {
         return 0;
     }
-    halide_assert(user_context, buf->device_interface == &cuda_device_interface);
+    halide_abort_if_false(user_context, buf->device_interface == &cuda_device_interface);
     return (uintptr_t)buf->device;
 }
 
