@@ -43,8 +43,8 @@ namespace {
 
 // In this function we pick the dma engine and assign it to a virtual engine
 inline void *hexagon_dma_pool_get(void *user_context, void *virtual_engine_id) {
-    halide_assert(user_context, hexagon_dma_pool);
-    halide_assert(user_context, virtual_engine_id);
+    halide_abort_if_false(user_context, hexagon_dma_pool);
+    halide_abort_if_false(user_context, virtual_engine_id);
     ScopedMutexLock lock(&hexagon_dma_pool_mutex);
 
     hexagon_dma_virtual_engine_t *virtual_engine_addr = (hexagon_dma_virtual_engine_t *)virtual_engine_id;
@@ -66,7 +66,7 @@ inline void *hexagon_dma_pool_get(void *user_context, void *virtual_engine_id) {
                 virtual_engine_addr->mapped_engines[virtual_engine_addr->num_of_engines] = j + 1;
                 if (!hexagon_dma_pool->dma_engine_list[j].engine_addr) {
                     hexagon_dma_pool->dma_engine_list[j].engine_addr = (void *)hDmaWrapper_AllocDma();
-                    halide_assert(user_context, hexagon_dma_pool->dma_engine_list[j].engine_addr);
+                    halide_abort_if_false(user_context, hexagon_dma_pool->dma_engine_list[j].engine_addr);
                 }
                 virtual_engine_addr->num_of_engines++;
                 return hexagon_dma_pool->dma_engine_list[j].engine_addr;
@@ -79,7 +79,7 @@ inline void *hexagon_dma_pool_get(void *user_context, void *virtual_engine_id) {
 
 // In this function we simply mark the dma engine as free
 inline int hexagon_dma_pool_put(void *user_context, void *dma_engine, void *virtual_engine_id) {
-    halide_assert(user_context, virtual_engine_id);
+    halide_abort_if_false(user_context, virtual_engine_id);
     ScopedMutexLock lock(&hexagon_dma_pool_mutex);
 
     hexagon_dma_virtual_engine_t *virtual_engine_addr = (hexagon_dma_virtual_engine_t *)virtual_engine_id;
@@ -100,8 +100,8 @@ extern "C" {
 
 // halide_hexagon_free_dma_resource
 WEAK int halide_hexagon_free_dma_resource(void *user_context, void *virtual_engine_id) {
-    halide_assert(user_context, hexagon_dma_pool);
-    halide_assert(user_context, virtual_engine_id);
+    halide_abort_if_false(user_context, hexagon_dma_pool);
+    halide_abort_if_false(user_context, virtual_engine_id);
     // Free the Real DMA Engines
     int nRet = halide_error_code_success;
 
