@@ -4,6 +4,8 @@
 #ifndef _QAIC_ENV_H
 #define _QAIC_ENV_H
 
+#include <stdio.h>
+
 #ifdef __GNUC__
 #ifdef __clang__
 #pragma GCC diagnostic ignored "-Wunknown-pragmas"
@@ -142,15 +144,16 @@ static __inline void _qaic_memmove(void* dst, void* src, int size) {
 #define _ASSERT(nErr, ff) _TRY(nErr, 0 == (ff) ? AEE_EBADPARM : AEE_SUCCESS)
 
 #ifdef __QAIC_DEBUG__
-#define _ALLOCATE(nErr, pal, size, alignment, pv) _TRY(nErr, _allocator_alloc(pal, __FILE_LINE__, size, alignment, (void**)&pv))
+#define _ALLOCATE(nErr, pal, size, alignment, pv) _TRY(nErr, _allocator_alloc(pal, __FILE_LINE__, size, alignment, (void**)&pv));\
+                                                  _ASSERT(nErr,pv || !(size))
 #else
-#define _ALLOCATE(nErr, pal, size, alignment, pv) _TRY(nErr, _allocator_alloc(pal, 0, size, alignment, (void**)&pv))
+#define _ALLOCATE(nErr, pal, size, alignment, pv) _TRY(nErr, _allocator_alloc(pal, 0, size, alignment, (void**)&pv));\
+                                                  _ASSERT(nErr,pv || !(size))
 #endif
 
 
 #endif // _QAIC_ENV_H
 
-#include "remote.h"
 #ifndef _ALLOCATOR_H
 #define _ALLOCATOR_H
 
@@ -408,7 +411,6 @@ struct Interface {
 
 #ifndef _HALIDE_HEXAGON_REMOTE_SLIM_H
 #define _HALIDE_HEXAGON_REMOTE_SLIM_H
-#include "remote.h"
 #include <stdint.h>
 
 #ifndef __QAIC_SLIM
@@ -418,14 +420,14 @@ struct Interface {
 #define __QAIC_SLIM_EXPORT
 #endif
 
-static const Type types[3];
-static const SequenceType sequenceTypes[1] = {{&(types[1]),0x0,0x4,0x4,0x0}};
-static const Type types[3] = {{0x1,{{(const uintptr_t)0,(const uintptr_t)0}}, 2,0x1},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[0]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8)},{0x8,{{(const uintptr_t)0,(const uintptr_t)0}}, 2,0x8}};
-static const Parameter parameters[9] = {{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[0]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8),0,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)0}}, 2,0x4,3,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)0}}, 2,0x4,0,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(sequenceTypes[0]),0}}, 25,SLIM_IFPTR32(0x4,0x8),0,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(sequenceTypes[0]),0}}, 25,SLIM_IFPTR32(0x4,0x8),3,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[2]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8),0,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[0]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8),3,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x4,3,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x4,0,0}};
-static const Parameter* const parameterArrays[23] = {(&(parameters[8])),(&(parameters[2])),(&(parameters[2])),(&(parameters[8])),(&(parameters[2])),(&(parameters[2])),(&(parameters[8])),(&(parameters[8])),(&(parameters[2])),(&(parameters[2])),(&(parameters[3])),(&(parameters[4])),(&(parameters[5])),(&(parameters[2])),(&(parameters[0])),(&(parameters[1])),(&(parameters[0])),(&(parameters[0])),(&(parameters[1])),(&(parameters[7])),(&(parameters[7])),(&(parameters[6])),(&(parameters[7]))};
-static const Method methods[9] = {{REMOTE_SCALARS_MAKEX(0,0,0x3,0x1,0x0,0x0),0x8,0x4,5,3,(&(parameterArrays[16])),0x4,0x4},{REMOTE_SCALARS_MAKEX(0,0,0x2,0x1,0x0,0x0),0x8,0x4,4,3,(&(parameterArrays[13])),0x4,0x4},{REMOTE_SCALARS_MAKEX(0,0,0x0,0x0,0x0,0x0),0x0,0x0,0,0,0,0x0,0x0},{REMOTE_SCALARS_MAKEX(0,0,255,255,15,15),0x14,0x0,9,5,(&(parameterArrays[8])),0x4,0x1},{REMOTE_SCALARS_MAKEX(0,0,0x1,0x0,0x0,0x0),0x4,0x0,1,1,(&(parameterArrays[1])),0x4,0x0},{REMOTE_SCALARS_MAKEX(0,0,0x1,0x2,0x0,0x0),0x4,0x4,4,2,(&(parameterArrays[21])),0x4,0x4},{REMOTE_SCALARS_MAKEX(0,0,0x0,0x1,0x0,0x0),0x0,0x8,2,2,(&(parameterArrays[19])),0x1,0x4},{REMOTE_SCALARS_MAKEX(0,0,0x1,0x0,0x0,0x0),0x4,0x0,1,1,(&(parameterArrays[0])),0x4,0x0},{REMOTE_SCALARS_MAKEX(0,0,0x1,0x0,0x0,0x0),0x20,0x0,8,8,(&(parameterArrays[0])),0x4,0x0}};
+static const Type types[4];
+static const SequenceType sequenceTypes[1] = {{&(types[2]),0x0,0x4,0x4,0x0}};
+static const Type types[4] = {{0x1,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x1},{0x1,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x1},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[1]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8)},{0x8,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x8}};
+static const Parameter parameters[11] = {{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[0]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8),0,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[1]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8),0,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x4,3,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x4,0,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(sequenceTypes[0]),0}}, 25,SLIM_IFPTR32(0x4,0x8),0,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(sequenceTypes[0]),0}}, 25,SLIM_IFPTR32(0x4,0x8),3,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[3]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8),0,0},{SLIM_IFPTR32(0x8,0x10),{{(const uintptr_t)&(types[0]),(const uintptr_t)0x0}}, 9,SLIM_IFPTR32(0x4,0x8),3,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x4,3,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x4,0,0},{0x4,{{(const uintptr_t)0,(const uintptr_t)1}}, 2,0x4,0,0}};
+static const Parameter* const parameterArrays[23] = {(&(parameters[9])),(&(parameters[10])),(&(parameters[10])),(&(parameters[9])),(&(parameters[10])),(&(parameters[10])),(&(parameters[9])),(&(parameters[9])),(&(parameters[3])),(&(parameters[3])),(&(parameters[4])),(&(parameters[5])),(&(parameters[6])),(&(parameters[3])),(&(parameters[0])),(&(parameters[2])),(&(parameters[0])),(&(parameters[1])),(&(parameters[2])),(&(parameters[8])),(&(parameters[8])),(&(parameters[7])),(&(parameters[8]))};
+static const Method methods[9] = {{REMOTE_SCALARS_MAKEX(0,0,0x3,0x1,0x0,0x0),0x8,0x4,5,3,(&(parameterArrays[16])),0x4,0x4},{REMOTE_SCALARS_MAKEX(0,0,0x2,0x1,0x0,0x0),0x8,0x4,4,3,(&(parameterArrays[13])),0x4,0x4},{REMOTE_SCALARS_MAKEX(0,0,0x0,0x0,0x0,0x0),0x0,0x0,0,0,0,0x0,0x0},{REMOTE_SCALARS_MAKEX(0,0,255,255,15,15),0x14,0x0,9,5,(&(parameterArrays[8])),0x4,0x1},{REMOTE_SCALARS_MAKEX(0,0,0x1,0x0,0x0,0x0),0x4,0x0,1,1,(&(parameterArrays[8])),0x4,0x0},{REMOTE_SCALARS_MAKEX(0,0,0x1,0x2,0x0,0x0),0x4,0x4,4,2,(&(parameterArrays[21])),0x4,0x4},{REMOTE_SCALARS_MAKEX(0,0,0x0,0x1,0x0,0x0),0x0,0x8,2,2,(&(parameterArrays[19])),0x1,0x4},{REMOTE_SCALARS_MAKEX(0,0,0x1,0x0,0x0,0x0),0x4,0x0,1,1,(&(parameterArrays[0])),0x4,0x0},{REMOTE_SCALARS_MAKEX(0,0,0x1,0x0,0x0,0x0),0x20,0x0,8,8,(&(parameterArrays[0])),0x4,0x0}};
 static const Method* const methodArrays[12] = {&(methods[0]),&(methods[1]),&(methods[2]),&(methods[2]),&(methods[3]),&(methods[4]),&(methods[5]),&(methods[6]),&(methods[7]),&(methods[7]),&(methods[8]),&(methods[7])};
-static const char strings[377] = "profiler_set_current_func\0busbwUsagePercentage\0set_performance_mode\0set_thread_priority\0poll_profiler_state\0bwMegabytesPerSec\0set_performance\0release_library\0output_buffers\0mipsPerThread\0input_buffers\0power_hvx_off\0get_symbol_v4\0power_hvx_on\0load_library\0set_latency\0set_bus_bw\0module_ptr\0mipsTotal\0read_size\0set_mips\0poll_log\0threads\0scalars\0sym_ptr\0symbol\0run_v2\0soname\0code\0";
+static const char strings[378] = "profiler_set_current_func\0busbwUsagePercentage\0set_performance_mode\0set_thread_priority\0poll_profiler_state\0bwMegabytesPerSec\0set_performance\0release_library\0output_buffers\0mipsPerThread\0input_buffers\0power_hvx_off\0get_symbol_v4\0power_hvx_on\0load_library\0set_latency\0set_bus_bw\0module_ptr\0mipsTotal\0read_size\0set_mips\0poll_log\0threads\0scalars\0sym_ptr\0symbol\0run_v2\0soname\0code\0";
 static const uint16_t methodStrings[39] = {126,309,173,289,267,108,26,255,259,358,278,351,187,158,335,215,278,367,343,242,365,372,278,88,21,327,318,323,299,68,79,47,63,0,13,142,278,201,229};
 static const uint16_t methodStringsArrays[12] = {19,15,38,37,9,35,26,23,33,31,0,29};
 __QAIC_SLIM_EXPORT const Interface __QAIC_SLIM(halide_hexagon_remote_slim) = {12,&(methodArrays[0]),0,0,&(methodStringsArrays [0]),methodStrings,strings};
@@ -482,7 +484,11 @@ __QAIC_STUB_EXPORT remote_handle _halide_hexagon_remote_handle(void) {
 uint32_t _halide_hexagon_remote_atomic_CompareAndExchange(uint32_t * volatile puDest, uint32_t uExchange, uint32_t uCompare);
 
 #ifdef _WIN32
+#ifdef _USRDLL
 #include "Windows.h"
+#else
+#include "ntddk.h"
+#endif //_USRDLL
 uint32_t _halide_hexagon_remote_atomic_CompareAndExchange(uint32_t * volatile puDest, uint32_t uExchange, uint32_t uCompare) {
    return (uint32_t)InterlockedCompareExchange((volatile LONG*)puDest, (LONG)uExchange, (LONG)uCompare);
 }
@@ -512,10 +518,6 @@ __QAIC_STUB_EXPORT remote_handle _halide_hexagon_remote_handle(void) {
 
 #endif //__qdsp6__
 
-__QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_skel_invoke)(uint32_t _sc, remote_arg* _pra) __QAIC_STUB_ATTRIBUTE {
-   return __QAIC_REMOTE(remote_handle_invoke)(_halide_hexagon_remote_handle(), _sc, _pra);
-}
-
 #ifdef __cplusplus
 }
 #endif
@@ -539,10 +541,10 @@ static __inline int _stub_method(remote_handle _handle, uint32_t _mid, char* _in
    _pra[(_numIn[0] + 1)].buf.nLen = sizeof(_primROut);
    _COPY(_primIn, 0, _in0Len, 0, 4);
    _praIn = (_pra + 1);
-   _praIn[0].buf.pv = _in0[0];
+   _praIn[0].buf.pv = (void*) _in0[0];
    _praIn[0].buf.nLen = (1 * _in0Len[0]);
    _COPY(_primIn, 4, _in1Len, 0, 4);
-   _praIn[1].buf.pv = _in1[0];
+   _praIn[1].buf.pv = (void*) _in1[0];
    _praIn[1].buf.nLen = (1 * _in1Len[0]);
    _TRY(_nErr, __QAIC_REMOTE(remote_handle_invoke)(_handle, REMOTE_SCALARS_MAKEX(0, _mid, 3, 1, 0, 0), _pra));
    _COPY(_rout2, 0, _primROut, 0, 4);
@@ -551,7 +553,12 @@ static __inline int _stub_method(remote_handle _handle, uint32_t _mid, char* _in
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_load_library)(const char* soname, int sonameLen, const unsigned char* code, int codeLen, halide_hexagon_remote_handle_t* module_ptr) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 0;
-   return _stub_method(_halide_hexagon_remote_handle(), _mid, (char**)&soname, (uint32_t*)&sonameLen, (char**)&code, (uint32_t*)&codeLen, (uint32_t*)module_ptr);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method(_handle, _mid, (char**)&soname, (uint32_t*)&sonameLen, (char**)&code, (uint32_t*)&codeLen, (uint32_t*)module_ptr);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 static __inline int _stub_method_1(remote_handle _handle, uint32_t _mid, uint32_t _in0[1], char* _in1[1], uint32_t _in1Len[1], uint32_t _rout2[1]) {
    int _numIn[1];
@@ -568,7 +575,7 @@ static __inline int _stub_method_1(remote_handle _handle, uint32_t _mid, uint32_
    _COPY(_primIn, 0, _in0, 0, 4);
    _COPY(_primIn, 4, _in1Len, 0, 4);
    _praIn = (_pra + 1);
-   _praIn[0].buf.pv = _in1[0];
+   _praIn[0].buf.pv = (void*) _in1[0];
    _praIn[0].buf.nLen = (1 * _in1Len[0]);
    _TRY(_nErr, __QAIC_REMOTE(remote_handle_invoke)(_handle, REMOTE_SCALARS_MAKEX(0, _mid, 2, 1, 0, 0), _pra));
    _COPY(_rout2, 0, _primROut, 0, 4);
@@ -577,7 +584,12 @@ static __inline int _stub_method_1(remote_handle _handle, uint32_t _mid, uint32_
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_get_symbol_v4)(halide_hexagon_remote_handle_t module_ptr, const char* name, int nameLen, halide_hexagon_remote_handle_t* sym_ptr) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 1;
-   return _stub_method_1(_halide_hexagon_remote_handle(), _mid, (uint32_t*)&module_ptr, (char**)&name, (uint32_t*)&nameLen, (uint32_t*)sym_ptr);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_1(_handle, _mid, (uint32_t*)&module_ptr, (char**)&name, (uint32_t*)&nameLen, (uint32_t*)sym_ptr);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 static __inline int _stub_method_2(remote_handle _handle, uint32_t _mid) {
    remote_arg* _pra = 0;
@@ -588,11 +600,21 @@ static __inline int _stub_method_2(remote_handle _handle, uint32_t _mid) {
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_power_hvx_on)(void) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 2;
-   return _stub_method_2(_halide_hexagon_remote_handle(), _mid);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_2(_handle, _mid);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_power_hvx_off)(void) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 3;
-   return _stub_method_2(_halide_hexagon_remote_handle(), _mid);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_2(_handle, _mid);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 static __inline int _stub_unpack(remote_arg* _praROutPost, remote_arg* _ppraROutPost[1], void* _primROut, char* _rout0[1], uint32_t _rout0Len[1]) {
    int _nErr = 0;
@@ -634,7 +656,7 @@ static __inline int _stub_pack_1(_allocator* _al, remote_arg* _praIn, remote_arg
    _ppraIn = &_praIn;
    _ppraROut = &_praROut;
    _COPY(_primIn, 0, _in0Len, 0, 4);
-   _praIn[0].buf.pv = _in0[0];
+   _praIn[0].buf.pv = (void*) _in0[0];
    _praIn[0].buf.nLen = (1 * _in0Len[0]);
    _ppraInStart[0] += (_praIn - _praInStart) + 1;
    _ppraROutStart[0] += (_praROut - _praROutStart) +0;
@@ -688,8 +710,13 @@ static __inline int _stub_method_3(remote_handle _handle, uint32_t _mid, uint32_
    {
       _count(_numIn, _numROut, _numInH, _numROutH, SLIM_IFPTR32((char**)&(((uint32_t*)_seq_nat3)[0]), (char**)&(((uint64_t*)_seq_nat3)[0])), SLIM_IFPTR32((uint32_t*)&(((uint32_t*)_seq_nat3)[1]), (uint32_t*)&(((uint32_t*)_seq_nat3)[2])));
    }
+   if(_numIn[0]>=255)
+      printf("ERROR: Unsupported number of input buffers\n");
+   if(_numROut[0]>=255)
+      printf("ERROR: Unsupported number of output buffers\n");
    _allocator_init(_al, 0, 0);
    _ALLOCATE(_nErr, _al, ((((((((_numIn[0] + _numROut[0]) + _numInH[0]) + _numROutH[0]) + 1) + 0) + 0) + 0) * sizeof(_pra[0])), 4, _pra);
+   _ASSERT(_nErr, _pra);
    _pra[0].buf.pv = (void*)_primIn;
    _pra[0].buf.nLen = sizeof(_primIn);
    _praIn = (_pra + 1);
@@ -718,7 +745,7 @@ static __inline int _stub_method_3(remote_handle _handle, uint32_t _mid, uint32_
       _TRY(_nErr, _stub_pack(_al, (_praIn + 2), _ppraIn, (_praROut + 0), _ppraROut, _praHIn, _ppraHIn, _praHROut, _ppraHROut, _seq_primIn3, 0, SLIM_IFPTR32((char**)&(((uint32_t*)_seq_nat3)[0]), (char**)&(((uint64_t*)_seq_nat3)[0])), SLIM_IFPTR32((uint32_t*)&(((uint32_t*)_seq_nat3)[1]), (uint32_t*)&(((uint32_t*)_seq_nat3)[2]))));
    }
    _COPY(_primIn, 16, _in4Len, 0, 4);
-   _praIn[2].buf.pv = _in4[0];
+   _praIn[2].buf.pv = (void*) _in4[0];
    _praIn[2].buf.nLen = (8 * _in4Len[0]);
    _ASSERT(_nErr, (_numInH[0] + 0) <= 15);
    _ASSERT(_nErr, (_numROutH[0] + 0) <= 15);
@@ -737,7 +764,12 @@ static __inline int _stub_method_3(remote_handle _handle, uint32_t _mid, uint32_
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_run_v2)(halide_hexagon_remote_handle_t module_ptr, halide_hexagon_remote_handle_t symbol, const halide_hexagon_remote_buffer* input_buffers, int input_buffersLen, halide_hexagon_remote_buffer* output_buffers, int output_buffersLen, const halide_hexagon_remote_scalar_t* scalars, int scalarsLen) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 4;
-   return _stub_method_3(_halide_hexagon_remote_handle(), _mid, (uint32_t*)&module_ptr, (uint32_t*)&symbol, (void**)&input_buffers, (uint32_t*)&input_buffersLen, (void**)&output_buffers, (uint32_t*)&output_buffersLen, (char**)&scalars, (uint32_t*)&scalarsLen);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_3(_handle, _mid, (uint32_t*)&module_ptr, (uint32_t*)&symbol, (void**)&input_buffers, (uint32_t*)&input_buffersLen, (void**)&output_buffers, (uint32_t*)&output_buffersLen, (char**)&scalars, (uint32_t*)&scalarsLen);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 static __inline int _stub_method_4(remote_handle _handle, uint32_t _mid, uint32_t _in0[1]) {
    remote_arg _pra[1];
@@ -752,7 +784,12 @@ static __inline int _stub_method_4(remote_handle _handle, uint32_t _mid, uint32_
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_release_library)(halide_hexagon_remote_handle_t module_ptr) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 5;
-   return _stub_method_4(_halide_hexagon_remote_handle(), _mid, (uint32_t*)&module_ptr);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_4(_handle, _mid, (uint32_t*)&module_ptr);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 static __inline int _stub_method_5(remote_handle _handle, uint32_t _mid, char* _rout0[1], uint32_t _rout0Len[1], uint32_t _rout1[1]) {
    int _numIn[1];
@@ -779,7 +816,12 @@ static __inline int _stub_method_5(remote_handle _handle, uint32_t _mid, char* _
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_poll_log)(char* log, int logLen, int* read_size) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 6;
-   return _stub_method_5(_halide_hexagon_remote_handle(), _mid, (char**)&log, (uint32_t*)&logLen, (uint32_t*)read_size);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_5(_handle, _mid, (char**)&log, (uint32_t*)&logLen, (uint32_t*)read_size);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 static __inline int _stub_method_6(remote_handle _handle, uint32_t _mid, uint32_t _rout0[1], uint32_t _rout1[1]) {
    int _numIn[1];
@@ -797,15 +839,30 @@ static __inline int _stub_method_6(remote_handle _handle, uint32_t _mid, uint32_
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_poll_profiler_state)(int* func, int* threads) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 7;
-   return _stub_method_6(_halide_hexagon_remote_handle(), _mid, (uint32_t*)func, (uint32_t*)threads);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_6(_handle, _mid, (uint32_t*)func, (uint32_t*)threads);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_profiler_set_current_func)(int current_func) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 8;
-   return _stub_method_4(_halide_hexagon_remote_handle(), _mid, (uint32_t*)&current_func);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_4(_handle, _mid, (uint32_t*)&current_func);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_set_performance_mode)(int mode) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 9;
-   return _stub_method_4(_halide_hexagon_remote_handle(), _mid, (uint32_t*)&mode);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_4(_handle, _mid, (uint32_t*)&mode);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 static __inline int _stub_method_7(remote_handle _handle, uint32_t _mid, uint32_t _in0[1], uint32_t _in1[1], uint32_t _in2[1], uint32_t _in3[1], uint32_t _in4[1], uint32_t _in5[1], uint32_t _in6[1], uint32_t _in7[1]) {
    remote_arg _pra[1];
@@ -827,11 +884,21 @@ static __inline int _stub_method_7(remote_handle _handle, uint32_t _mid, uint32_
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_set_performance)(int set_mips, unsigned int mipsPerThread, unsigned int mipsTotal, int set_bus_bw, unsigned int bwMegabytesPerSec, unsigned int busbwUsagePercentage, int set_latency, int latency) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 10;
-   return _stub_method_7(_halide_hexagon_remote_handle(), _mid, (uint32_t*)&set_mips, (uint32_t*)&mipsPerThread, (uint32_t*)&mipsTotal, (uint32_t*)&set_bus_bw, (uint32_t*)&bwMegabytesPerSec, (uint32_t*)&busbwUsagePercentage, (uint32_t*)&set_latency, (uint32_t*)&latency);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_7(_handle, _mid, (uint32_t*)&set_mips, (uint32_t*)&mipsPerThread, (uint32_t*)&mipsTotal, (uint32_t*)&set_bus_bw, (uint32_t*)&bwMegabytesPerSec, (uint32_t*)&busbwUsagePercentage, (uint32_t*)&set_latency, (uint32_t*)&latency);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 __QAIC_STUB_EXPORT int __QAIC_STUB(halide_hexagon_remote_set_thread_priority)(int priority) __QAIC_STUB_ATTRIBUTE {
    uint32_t _mid = 11;
-   return _stub_method_4(_halide_hexagon_remote_handle(), _mid, (uint32_t*)&priority);
+   remote_handle _handle = _halide_hexagon_remote_handle();
+   if (_handle != -1)
+   return _stub_method_4(_handle, _mid, (uint32_t*)&priority);
+   else {
+   return AEE_EBADHANDLE;
+    }
 }
 #ifdef __cplusplus
 }
