@@ -268,7 +268,7 @@ WEAK int halide_openglcompute_device_malloc(void *user_context, halide_buffer_t
     }
 
     size_t size = buf->size_in_bytes();
-    halide_assert(user_context, size != 0);
+    halide_abort_if_false(user_context, size != 0);
 
     if (buf->device) {
         // This buffer already has a device allocation
@@ -278,7 +278,7 @@ WEAK int halide_openglcompute_device_malloc(void *user_context, halide_buffer_t
     }
 
     for (int i = 0; i < buf->dimensions; i++) {
-        halide_assert(user_context, buf->dim[i].stride >= 0);
+        halide_abort_if_false(user_context, buf->dim[i].stride >= 0);
     }
 
     debug(user_context) << "    allocating buffer, "
@@ -314,7 +314,7 @@ WEAK int halide_openglcompute_device_malloc(void *user_context, halide_buffer_t
     // types, all of which are 4 bytes. We'll inflate the size for
     // smaller types.
     size *= (4 / buf->type.bytes());
-    halide_assert(user_context, size != 0);
+    halide_abort_if_false(user_context, size != 0);
     global_state.BufferData(GL_ARRAY_BUFFER, size, nullptr, GL_DYNAMIC_COPY);
     if (global_state.CheckAndReportError(user_context, "oglc: BufferData")) {
         return 1;
@@ -499,7 +499,7 @@ WEAK int halide_openglcompute_copy_to_host(void *user_context, halide_buffer_t *
 
     GLuint the_buffer = (GLuint)buf->device;
     size_t size = buf->size_in_bytes();
-    halide_assert(user_context, size != 0);
+    halide_abort_if_false(user_context, size != 0);
 
     debug(user_context) << "OGLC: halide_openglcompute_copy_to_host ("
                         << "user_context: " << user_context
@@ -755,6 +755,7 @@ WEAK char *get_kernel_name(const char *start, const char *end) {
 WEAK int halide_openglcompute_initialize_kernels(void *user_context, void **state_ptr,
                                                  const char *src, int size) {
 #ifdef DEBUG_RUNTIME
+    halide_start_clock(user_context);
     uint64_t t_before = halide_current_time_ns(user_context);
 #endif
 
