@@ -261,10 +261,10 @@ private:
         std::string file_name = output_directory + fn_name;
 
         auto ext = Internal::get_output_info(target);
-        std::map<Output, std::string> outputs = {
-            {Output::c_header, file_name + ext.at(Output::c_header).extension},
-            {Output::object, file_name + ext.at(Output::object).extension},
-            {Output::assembly, file_name + ".s"},
+        std::map<OutputFileType, std::string> outputs = {
+            {OutputFileType::c_header, file_name + ext.at(OutputFileType::c_header).extension},
+            {OutputFileType::object, file_name + ext.at(OutputFileType::object).extension},
+            {OutputFileType::assembly, file_name + ".s"},
         };
         error.compile_to(outputs, arg_types, fn_name, target);
 
@@ -275,7 +275,9 @@ private:
 
         string suffix = suffix_map[name];
         std::ostringstream msg;
-        msg << op << " did not generate for target=" << target.to_string() << " suffix=" << suffix << " vector_width=" << vector_width << ". Instead we got:\n";
+        msg << op << " did not generate for target=" << target.to_string()
+            << " suffix=" << suffix
+            << " vector_width=" << vector_width << ". Instead we got:\n";
 
         string line;
         while (getline(asm_file, line)) {
@@ -374,4 +376,4 @@ int main(int argc, char **argv) {
 
     printf("Success!\n");
     return 0;
-}
\ No newline at end of file
+}
