@@ -7,7 +7,7 @@ using namespace Halide;
 
 Internal::GpuObjectLifetimeTracker tracker;
 
-void halide_print(void *user_context, const char *str) {
+void halide_print(JITUserContext *user_context, const char *str) {
     printf("%s", str);
 
     tracker.record_gpu_debug(str);
@@ -16,12 +16,13 @@ void halide_print(void *user_context, const char *str) {
 int main(int argc, char *argv[]) {
     Var x, xi;
 
-    Internal::JITHandlers handlers;
+    Target target = get_jit_target_from_environment();
+
+    // We need to hook the default handler too, to catch the frees done by release_all
+    JITHandlers handlers;
     handlers.custom_print = halide_print;
     Internal::JITSharedRuntime::set_default_handlers(handlers);
 
-    Target target = get_jit_target_from_environment();
-
     // We need debug output to record object creation.
     target.set_feature(Target::Debug);
 
@@ -34,7 +35,6 @@ int main(int argc, char *argv[]) {
         } else if (target.has_feature(Target::HVX)) {
             f.hexagon();
         }
-        f.set_custom_print(halide_print);
 
         Buffer<int32_t> result = f.realize({256}, target);
         for (int i = 0; i < 256; i++) {
