@@ -6,7 +6,7 @@ using namespace Halide;
 bool vector_store = false, scalar_store = false;
 
 // A trace that checks for vector and scalar stores
-int my_trace(void *user_context, const halide_trace_event_t *ev) {
+int my_trace(JITUserContext *user_context, const halide_trace_event_t *ev) {
 
     if (ev->event == halide_trace_store) {
         if (ev->type.lanes > 1) {
@@ -23,7 +23,7 @@ int main(int argc, char **argv) {
     Var x;
 
     f(x) = x;
-    f.set_custom_trace(&my_trace);
+    f.jit_handlers().custom_trace = &my_trace;
     f.trace_stores();
 
     Buffer<int> result_1 = f.realize({10});
