@@ -9,7 +9,7 @@ using namespace Halide;
 const int tolerance = 3 * sizeof(int);
 std::set<size_t> custom_malloc_sizes;
 
-void *my_malloc(void *user_context, size_t x) {
+void *my_malloc(JITUserContext *user_context, size_t x) {
     custom_malloc_sizes.insert(x);
     void *orig = malloc(x + 32);
     void *ptr = (void *)((((size_t)orig + 32) >> 5) << 5);
@@ -17,7 +17,7 @@ void *my_malloc(void *user_context, size_t x) {
     return ptr;
 }
 
-void my_free(void *user_context, void *ptr) {
+void my_free(JITUserContext *user_context, void *ptr) {
     free(((void **)ptr)[-1]);
 }
 
@@ -96,7 +96,7 @@ extern "C" DLLEXPORT int zigzag_buffer_copy(halide_buffer_t *in, halide_buffer_t
 }
 
 bool error_occurred;
-void expected_error(void *, const char *msg) {
+void expected_error(JITUserContext *, const char *msg) {
     // Emitting "error.*:" to stdout or stderr will cause CMake to report the
     // test as a failure on Windows, regardless of error code returned,
     // hence the abbreviation to "err".
@@ -106,7 +106,7 @@ void expected_error(void *, const char *msg) {
 
 void realize_and_expect_error(Func f, int w, int h) {
     error_occurred = false;
-    f.set_error_handler(expected_error);
+    f.jit_handlers().custom_error = expected_error;
     f.realize({w, h});
     if (!error_occurred) {
         printf("Expected an error!\n");
@@ -116,12 +116,18 @@ void realize_and_expect_error(Func f, int w, int h) {
 
 int main(int argc, char **argv) {
     if (get_jit_target_from_environment().arch == Target::WebAssembly) {
-        printf("[SKIP] WebAssembly JIT does not support set_custom_allocator().\n");
+        printf("[SKIP] WebAssembly JIT does not support custom allocators.\n");
         return 0;
     }
 
     Var x, y, c;
 
+    // Every allocation in this test wants to go through the custom allocator above.
+    JITHandlers handlers;
+    handlers.custom_malloc = my_malloc;
+    handlers.custom_free = my_free;
+    Internal::JITSharedRuntime::set_default_handlers(handlers);
+
     {
         Func f, g;
 
@@ -131,8 +137,6 @@ int main(int argc, char **argv) {
 
         // Should be able to fold storage in y and c
 
-        g.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = g.realize({100, 1000, 3});
 
         size_t expected_size = 101 * 4 * sizeof(int);
@@ -152,8 +156,6 @@ int main(int argc, char **argv) {
         // Make sure that storage folding doesn't happen if there are
         // multiple producers of the folded buffer.
 
-        g.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = g.realize({100, 1000, 3});
 
         size_t expected_size = 101 * 1002 * 3 * sizeof(int);
@@ -175,8 +177,6 @@ int main(int argc, char **argv) {
         // automatic storage folding refused to fold this (the case
         // above).
 
-        g.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = g.realize({100, 1000});
 
         size_t expected_size = 101 * 3 * sizeof(int);
@@ -197,8 +197,6 @@ int main(int argc, char **argv) {
         // allocation.
         g.compute_at(f, x).store_root();
 
-        f.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = f.realize({1000, 1000});
 
         if (!custom_malloc_sizes.empty()) {
@@ -230,8 +228,6 @@ int main(int argc, char **argv) {
 
         g.compute_at(f, x).store_root();
 
-        f.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = f.realize({1000, 1000});
 
         if (!custom_malloc_sizes.empty()) {
@@ -264,8 +260,6 @@ int main(int argc, char **argv) {
 
         g.compute_at(f, x).store_root();
 
-        f.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = f.realize({1000, 1000});
 
         size_t expected_size = 2 * 1000 * 4 * sizeof(int);
@@ -300,8 +294,6 @@ int main(int argc, char **argv) {
         // correct to fold if we know the output height is a multiple
         // of the split factor.
 
-        f.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = f.realize({1000, 1000});
 
         size_t expected_size = 1000 * 8 * sizeof(int);
@@ -335,8 +327,6 @@ int main(int argc, char **argv) {
         // (e.g. if memory usage is a concern.)
         g.compute_at(f, x).store_root().fold_storage(y, 3);
 
-        f.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = f.realize({1000, 1000});
 
         size_t expected_size = 2 * 1000 * 3 * sizeof(int);
@@ -365,8 +355,6 @@ int main(int argc, char **argv) {
 
         g.compute_at(f, x).store_root();
 
-        f.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = f.realize({1000, 1000});
 
         size_t expected_size = 1000 * 2 * sizeof(int);
@@ -397,8 +385,6 @@ int main(int argc, char **argv) {
         h.compute_at(f, y).store_root().fold_storage(y, 4);
         g.compute_at(f, y).store_root().fold_storage(y, 2);
 
-        f.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = f.realize({1000, 1000});
 
         // Halide allocates one extra scalar, so we account for that.
@@ -436,8 +422,6 @@ int main(int argc, char **argv) {
         // Make sure we can explicitly fold something with an outer
         // loop.
 
-        g.set_custom_allocator(my_malloc, my_free);
-
         Buffer<int> im = g.realize({100, 1000, 3});
 
         size_t expected_size;
