@@ -4,7 +4,7 @@ using namespace Halide;
 
 int num_vector_stores = 0;
 int num_scalar_stores = 0;
-int my_trace(void *user_context, const halide_trace_event_t *e) {
+int my_trace(JITUserContext *user_context, const halide_trace_event_t *e) {
     if (e->event == halide_trace_store) {
         if (e->type.lanes > 1) {
             num_vector_stores++;
@@ -27,7 +27,7 @@ int main(int argc, char **argv) {
         const int expected_vector_stores = w / v;
         const int expected_scalar_stores = w % v;
 
-        f.set_custom_trace(&my_trace);
+        f.jit_handlers().custom_trace = &my_trace;
         f.trace_stores();
 
         num_vector_stores = 0;
