@@ -5,8 +5,8 @@ namespace {
 class Alias : public Halide::Generator<Alias> {
 public:
     GeneratorParam<int32_t> offset{"offset", 0};
-    Input<Buffer<int32_t>> input{"input", 1};
-    Output<Buffer<int32_t>> output{"output", 1};
+    Input<Buffer<int32_t, 1>> input{"input"};
+    Output<Buffer<int32_t, 1>> output{"output"};
 
     void generate() {
         Var x;
