@@ -10,7 +10,7 @@ using namespace Halide::Runtime;
 
 const int kSize = 32;
 
-void verify(const Buffer<int32_t> &img, float f1, float f2) {
+void verify(const Buffer<int32_t, 3> &img, float f1, float f2) {
     for (int i = 0; i < kSize; i++) {
         for (int j = 0; j < kSize; j++) {
             for (int c = 0; c < 3; c++) {
@@ -27,7 +27,7 @@ void verify(const Buffer<int32_t> &img, float f1, float f2) {
 int main(int argc, char **argv) {
 
     int result;
-    Buffer<int32_t> output(kSize, kSize, 3);
+    Buffer<int32_t, 3> output(kSize, kSize, 3);
 
     // We can, of course, pass whatever values for Param/ImageParam that we like.
     result = argvcall(1.2f, 3.4f, output);
