@@ -10,7 +10,7 @@ using namespace Halide::Runtime;
 
 const int kSize = 32;
 
-void verify(const Buffer<int32_t> &img, float compiletime_factor, float runtime_factor, int channels) {
+void verify(const Buffer<int32_t, 3> &img, float compiletime_factor, float runtime_factor, int channels) {
     img.for_each_element([=](int x, int y, int c) {
         int expected = (int32_t)(compiletime_factor * runtime_factor * c * (x > y ? x : y));
         int actual = img(x, y, c);
@@ -20,7 +20,7 @@ void verify(const Buffer<int32_t> &img, float compiletime_factor, float runtime_
 
 int main(int argc, char **argv) {
 
-    Buffer<int32_t> output(kSize, kSize, 3);
+    Buffer<int32_t, 3> output(kSize, kSize, 3);
 
     // For Ahead-of-time compilation, we don't get to customize any GeneratorParams:
     // they were baked into the object code by our build system. These are the default values
