@@ -29,7 +29,7 @@ int main(int argc, char **argv) {
 
     halide_thread *t = halide_spawn_thread(&mess_with_num_threads, nullptr);
 
-    Buffer<float> out(64, 64);
+    Buffer<float, 2> out(64, 64);
 
     for (int i = 0; i < 1000; i++) {
         // The number of threads will oscillate randomly, but the range
