@@ -62,11 +62,6 @@ struct FormatInfo {
 
 namespace Internal {
 
-// Must be constexpr to allow use in case clauses.
-inline constexpr int halide_type_code(halide_type_code_t code, int bits) {
-    return (((int)code) << 8) | bits;
-}
-
 typedef bool (*CheckFunc)(bool condition, const char *msg);
 
 inline bool CheckFail(bool condition, const char *msg) {
@@ -699,11 +694,13 @@ struct FileOpener {
     FILE *const f;
 };
 
+constexpr int AnyDims = -1;
+
 // Read a row of ElemTypes from a byte buffer and copy them into a specific image row.
 // Multibyte elements are assumed to be big-endian.
 template<typename ElemType, typename ImageType>
 void read_big_endian_row(const uint8_t *src, int y, ImageType *im) {
-    auto im_typed = im->template as<ElemType>();
+    auto im_typed = im->template as<ElemType, AnyDims>();
     const int xmin = im_typed.dim(0).min();
     const int xmax = im_typed.dim(0).max();
     if (im_typed.dimensions() > 2) {
@@ -727,7 +724,7 @@ void read_big_endian_row(const uint8_t *src, int y, ImageType *im) {
 // Multibyte elements are written in big-endian layout.
 template<typename ElemType, typename ImageType>
 void write_big_endian_row(const ImageType &im, int y, uint8_t *dst) {
-    auto im_typed = im.template as<typename std::add_const<ElemType>::type>();
+    auto im_typed = im.template as<typename std::add_const<ElemType>::type, AnyDims>();
     const int xmin = im_typed.dim(0).min();
     const int xmax = im_typed.dim(0).max();
     if (im_typed.dimensions() > 2) {
@@ -1922,17 +1919,17 @@ bool save_tiff(ImageType &im, const std::string &filename) {
     }
 
     // Otherwise, write it out via manual traversal.
-#define HANDLE_CASE(CODE, BITS, TYPE)                    \
-    case halide_type_code(CODE, BITS): {                 \
-        ElemWriter<TYPE> ew(&f);                         \
-        im.template as<const TYPE>().for_each_value(ew); \
-        if (!check(ew.ok, "TIFF write failed")) {        \
-            return false;                                \
-        }                                                \
-        break;                                           \
+#define HANDLE_CASE(CODE, BITS, TYPE)                             \
+    case halide_type_t(CODE, BITS).as_u32(): {                    \
+        ElemWriter<TYPE> ew(&f);                                  \
+        im.template as<const TYPE, AnyDims>().for_each_value(ew); \
+        if (!check(ew.ok, "TIFF write failed")) {                 \
+            return false;                                         \
+        }                                                         \
+        break;                                                    \
     }
 
-    switch (halide_type_code((halide_type_code_t)im_type.code, im_type.bits)) {
+    switch (im_type.element_of().as_u32()) {
         HANDLE_CASE(halide_type_float, 32, float)
         HANDLE_CASE(halide_type_float, 64, double)
         HANDLE_CASE(halide_type_int, 8, int8_t)
@@ -1954,16 +1951,22 @@ bool save_tiff(ImageType &im, const std::string &filename) {
     return true;
 }
 
-// Given something like ImageType<Foo>, produce typedef ImageType<Bar>
-template<typename ImageType, typename ElemType>
-struct ImageTypeWithElemType {
-    using type = decltype(std::declval<ImageType>().template as<ElemType>());
+// Given something like ImageType<Foo, 2>, produce typedef ImageType<Foo, AnyDims>
+template<typename ImageType>
+struct ImageTypeWithDynamicDims {
+    using type = decltype(std::declval<ImageType>().template as<typename ImageType::ElemType, AnyDims>());
 };
 
-// Given something like ImageType<Foo>, produce typedef ImageType<const Bar>
+// Given something like ImageType<Foo>, produce typedef ImageType<Bar, AnyDims>
+template<typename ImageType, typename ElemType>
+struct ImageTypeWithElemType {
+    using type = decltype(std::declval<ImageType>().template as<ElemType, AnyDims>());
+};
+
+// Given something like ImageType<Foo>, produce typedef ImageType<const Bar, AnyDims>
 template<typename ImageType, typename ElemType>
 struct ImageTypeWithConstElemType {
-    using type = decltype(std::declval<ImageType>().template as<typename std::add_const<ElemType>::type>());
+    using type = decltype(std::declval<ImageType>().template as<typename std::add_const<ElemType>::type, AnyDims>());
 };
 
 template<typename ImageType, Internal::CheckFunc check>
@@ -2087,31 +2090,32 @@ struct ImageTypeConversion {
         // as documentation and a backstop against breakage.
         static_assert(!ImageType::has_static_halide_type,
                       "This variant of convert_image() requires a dynamically-typed image");
+        constexpr int AnyDims = Internal::AnyDims;
 
         const halide_type_t src_type = src.type();
-        switch (Internal::halide_type_code((halide_type_code_t)src_type.code, src_type.bits)) {
-        case Internal::halide_type_code(halide_type_float, 32):
-            return convert_image<DstElemType>(src.template as<float>());
-        case Internal::halide_type_code(halide_type_float, 64):
-            return convert_image<DstElemType>(src.template as<double>());
-        case Internal::halide_type_code(halide_type_int, 8):
-            return convert_image<DstElemType>(src.template as<int8_t>());
-        case Internal::halide_type_code(halide_type_int, 16):
-            return convert_image<DstElemType>(src.template as<int16_t>());
-        case Internal::halide_type_code(halide_type_int, 32):
-            return convert_image<DstElemType>(src.template as<int32_t>());
-        case Internal::halide_type_code(halide_type_int, 64):
-            return convert_image<DstElemType>(src.template as<int64_t>());
-        case Internal::halide_type_code(halide_type_uint, 1):
-            return convert_image<DstElemType>(src.template as<bool>());
-        case Internal::halide_type_code(halide_type_uint, 8):
-            return convert_image<DstElemType>(src.template as<uint8_t>());
-        case Internal::halide_type_code(halide_type_uint, 16):
-            return convert_image<DstElemType>(src.template as<uint16_t>());
-        case Internal::halide_type_code(halide_type_uint, 32):
-            return convert_image<DstElemType>(src.template as<uint32_t>());
-        case Internal::halide_type_code(halide_type_uint, 64):
-            return convert_image<DstElemType>(src.template as<uint64_t>());
+        switch (src_type.element_of().as_u32()) {
+        case halide_type_t(halide_type_float, 32).as_u32():
+            return convert_image<DstElemType>(src.template as<float, AnyDims>());
+        case halide_type_t(halide_type_float, 64).as_u32():
+            return convert_image<DstElemType>(src.template as<double, AnyDims>());
+        case halide_type_t(halide_type_int, 8).as_u32():
+            return convert_image<DstElemType>(src.template as<int8_t, AnyDims>());
+        case halide_type_t(halide_type_int, 16).as_u32():
+            return convert_image<DstElemType>(src.template as<int16_t, AnyDims>());
+        case halide_type_t(halide_type_int, 32).as_u32():
+            return convert_image<DstElemType>(src.template as<int32_t, AnyDims>());
+        case halide_type_t(halide_type_int, 64).as_u32():
+            return convert_image<DstElemType>(src.template as<int64_t, AnyDims>());
+        case halide_type_t(halide_type_uint, 1).as_u32():
+            return convert_image<DstElemType>(src.template as<bool, AnyDims>());
+        case halide_type_t(halide_type_uint, 8).as_u32():
+            return convert_image<DstElemType>(src.template as<uint8_t, AnyDims>());
+        case halide_type_t(halide_type_uint, 16).as_u32():
+            return convert_image<DstElemType>(src.template as<uint16_t, AnyDims>());
+        case halide_type_t(halide_type_uint, 32).as_u32():
+            return convert_image<DstElemType>(src.template as<uint32_t, AnyDims>());
+        case halide_type_t(halide_type_uint, 64).as_u32():
+            return convert_image<DstElemType>(src.template as<uint64_t, AnyDims>());
         default:
             assert(false && "Unsupported type");
             using DstImageType = typename Internal::ImageTypeWithElemType<ImageType, DstElemType>::type;
@@ -2134,32 +2138,33 @@ struct ImageTypeConversion {
 
         // Call the appropriate static-to-static conversion routine
         // based on the desired dst type.
-        switch (Internal::halide_type_code((halide_type_code_t)dst_type.code, dst_type.bits)) {
-        case Internal::halide_type_code(halide_type_float, 32):
+        switch (dst_type.element_of().as_u32()) {
+        case halide_type_t(halide_type_float, 32).as_u32():
             return convert_image<float>(src);
-        case Internal::halide_type_code(halide_type_float, 64):
+        case halide_type_t(halide_type_float, 64).as_u32():
             return convert_image<double>(src);
-        case Internal::halide_type_code(halide_type_int, 8):
+        case halide_type_t(halide_type_int, 8).as_u32():
             return convert_image<int8_t>(src);
-        case Internal::halide_type_code(halide_type_int, 16):
+        case halide_type_t(halide_type_int, 16).as_u32():
             return convert_image<int16_t>(src);
-        case Internal::halide_type_code(halide_type_int, 32):
+        case halide_type_t(halide_type_int, 32).as_u32():
             return convert_image<int32_t>(src);
-        case Internal::halide_type_code(halide_type_int, 64):
+        case halide_type_t(halide_type_int, 64).as_u32():
             return convert_image<int64_t>(src);
-        case Internal::halide_type_code(halide_type_uint, 1):
+        case halide_type_t(halide_type_uint, 1).as_u32():
             return convert_image<bool>(src);
-        case Internal::halide_type_code(halide_type_uint, 8):
+        case halide_type_t(halide_type_uint, 8).as_u32():
             return convert_image<uint8_t>(src);
-        case Internal::halide_type_code(halide_type_uint, 16):
+        case halide_type_t(halide_type_uint, 16).as_u32():
             return convert_image<uint16_t>(src);
-        case Internal::halide_type_code(halide_type_uint, 32):
+        case halide_type_t(halide_type_uint, 32).as_u32():
             return convert_image<uint32_t>(src);
-        case Internal::halide_type_code(halide_type_uint, 64):
+        case halide_type_t(halide_type_uint, 64).as_u32():
             return convert_image<uint64_t>(src);
         default:
             assert(false && "Unsupported type");
-            return ImageType();
+            using RetImageType = typename Internal::ImageTypeWithDynamicDims<ImageType>::type;
+            return RetImageType();
         }
     }
 
@@ -2175,38 +2180,40 @@ struct ImageTypeConversion {
         // as documentation and a backstop against breakage.
         static_assert(!ImageType::has_static_halide_type,
                       "This variant of convert_image() requires a dynamically-typed image");
+        constexpr int AnyDims = Internal::AnyDims;
 
         // Sniff the runtime type of src, coerce it to that type using as<>(),
         // and call the static-to-dynamic variant of this method. (Note that
         // this forces instantiation of the complete any-to-any conversion
         // matrix of code.)
         const halide_type_t src_type = src.type();
-        switch (Internal::halide_type_code((halide_type_code_t)src_type.code, src_type.bits)) {
-        case Internal::halide_type_code(halide_type_float, 32):
-            return convert_image(src.template as<float>(), dst_type);
-        case Internal::halide_type_code(halide_type_float, 64):
-            return convert_image(src.template as<double>(), dst_type);
-        case Internal::halide_type_code(halide_type_int, 8):
-            return convert_image(src.template as<int8_t>(), dst_type);
-        case Internal::halide_type_code(halide_type_int, 16):
-            return convert_image(src.template as<int16_t>(), dst_type);
-        case Internal::halide_type_code(halide_type_int, 32):
-            return convert_image(src.template as<int32_t>(), dst_type);
-        case Internal::halide_type_code(halide_type_int, 64):
-            return convert_image(src.template as<int64_t>(), dst_type);
-        case Internal::halide_type_code(halide_type_uint, 1):
-            return convert_image(src.template as<bool>(), dst_type);
-        case Internal::halide_type_code(halide_type_uint, 8):
-            return convert_image(src.template as<uint8_t>(), dst_type);
-        case Internal::halide_type_code(halide_type_uint, 16):
-            return convert_image(src.template as<uint16_t>(), dst_type);
-        case Internal::halide_type_code(halide_type_uint, 32):
-            return convert_image(src.template as<uint32_t>(), dst_type);
-        case Internal::halide_type_code(halide_type_uint, 64):
-            return convert_image(src.template as<uint64_t>(), dst_type);
+        switch (src_type.element_of().as_u32()) {
+        case halide_type_t(halide_type_float, 32).as_u32():
+            return convert_image(src.template as<float, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_float, 64).as_u32():
+            return convert_image(src.template as<double, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_int, 8).as_u32():
+            return convert_image(src.template as<int8_t, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_int, 16).as_u32():
+            return convert_image(src.template as<int16_t, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_int, 32).as_u32():
+            return convert_image(src.template as<int32_t, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_int, 64).as_u32():
+            return convert_image(src.template as<int64_t, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_uint, 1).as_u32():
+            return convert_image(src.template as<bool, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_uint, 8).as_u32():
+            return convert_image(src.template as<uint8_t, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_uint, 16).as_u32():
+            return convert_image(src.template as<uint16_t, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_uint, 32).as_u32():
+            return convert_image(src.template as<uint32_t, AnyDims>(), dst_type);
+        case halide_type_t(halide_type_uint, 64).as_u32():
+            return convert_image(src.template as<uint64_t, AnyDims>(), dst_type);
         default:
             assert(false && "Unsupported type");
-            return ImageType();
+            using RetImageType = typename Internal::ImageTypeWithDynamicDims<ImageType>::type;
+            return RetImageType();
         }
     }
 };
@@ -2235,7 +2242,7 @@ bool load(const std::string &filename, ImageType *im) {
             return false;
         }
     }
-    *im = im_d.template as<typename ImageType::ElemType>();
+    *im = im_d.template as<typename ImageType::ElemType, Internal::AnyDims>();
     im->set_host_dirty();
     return true;
 }
@@ -2256,7 +2263,7 @@ bool save(ImageType &im, const std::string &filename) {
 
     // Allow statically-typed images to be passed in, but quietly pass them on
     // as dynamically-typed images.
-    auto im_d = im.template as<const void>();
+    auto im_d = im.template as<const void, Internal::AnyDims>();
     return imageio.save(im_d, filename);
 }
 
@@ -2297,7 +2304,7 @@ public:
         Internal::CheckFail(ImageType::can_convert_from(im_d),
                             "Type mismatch assigning the result of load_image. "
                             "Did you mean to use load_and_convert_image?");
-        return im_d.template as<typename ImageType::ElemType>();
+        return im_d.template as<typename ImageType::ElemType, Internal::AnyDims>();
     }
 
 private:
@@ -2319,7 +2326,7 @@ public:
         (void)load<DynamicImageType, Internal::CheckFail>(filename, &im_d);
         const halide_type_t expected_type = ImageType::static_halide_type();
         if (im_d.type() == expected_type) {
-            return im_d.template as<typename ImageType::ElemType>();
+            return im_d.template as<typename ImageType::ElemType, Internal::AnyDims>();
         } else {
             return ImageTypeConversion::convert_image<typename ImageType::ElemType>(im_d);
         }
@@ -2340,7 +2347,8 @@ private:
 // a runtime error will occur.
 template<typename ImageType, Internal::CheckFunc check = Internal::CheckFail>
 void save_image(ImageType &im, const std::string &filename) {
-    (void)save<ImageType, check>(im, filename);
+    auto im_d = im.template as<void, Internal::AnyDims>();
+    (void)save<decltype(im_d), check>(im_d, filename);
 }
 
 // Like save_image, but quietly convert the saved image to a type that the
@@ -2352,11 +2360,12 @@ void convert_and_save_image(ImageType &im, const std::string &filename) {
     im.copy_to_host();
 
     std::set<FormatInfo> info;
-    (void)save_query<ImageType, check>(filename, &info);
+    (void)save_query<typename Internal::ImageTypeWithDynamicDims<ImageType>::type, check>(filename, &info);
     const FormatInfo best = Internal::best_save_format(im, info);
     if (best.type == im.type() && best.dimensions == im.dimensions()) {
         // It's an exact match, we can save as-is.
-        (void)save<ImageType, check>(im, filename);
+        using DynamicImageDims = typename Internal::ImageTypeWithDynamicDims<ImageType>::type;
+        (void)save<DynamicImageDims, check>(im.template as<typename ImageType::ElemType, Internal::AnyDims>(), filename);
     } else {
         using DynamicImageType = typename Internal::ImageTypeWithElemType<ImageType, void>::type;
         DynamicImageType im_converted = ImageTypeConversion::convert_image(im, best.type);
